<?php

namespace App\Controller;

use App\Entity\GP\TJornada;
use App\Entity\GP\TPersonalalta;

use App\Entity\TModalidad;

use App\Entity\TTitulacionMaestro;
use App\Entity\TAsignaturaCurso;

use App\Entity\TDepartamento;
use App\Entity\TFacultad;
use App\Entity\Tusuarios;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;
use App\ORM\ORM;
use Psr\Log\LoggerInterface;

use Symfony\Component\HttpFoundation\StreamedResponse;
use League\Csv\Writer;
use Dompdf\Dompdf;
use Dompdf\Options;
use Svg\Tag\Rect;


use Symfony\Component\Mailer\MailerInterface;

/**
 * @Route("/asignarDocente")
 */
class asignarDocenteController extends AbstractController
{

    private $orm;
    private $logger;
    private $entidad;

    public function __construct(EntityManagerInterface $entityManager, LoggerInterface $logger)
    {
        $this->orm = new ORM($entityManager);
        $this->logger = $logger;
        $this->entidad = TAsignaturaCurso::K_ENTIDAD;
    }

    /**
     * Pantalla inicial 
     * @Route("/", name="asignarDocente_index", methods={"GET"})
     */
    public function index(Request $request): Response
    {
        //comprobamos si hay errores
        $sError = $request->getSession()->get('error', NULL);

        //inicializar el error
        $request->getSession()->set('error', NULL);

        //datos del usuario conectado
        $sNombre = $request->getSession()->get('usuarioAplicativo', NULL);
        $oUsuario = $request->getSession()->get('usuarioApp', null);
        $nIdAcada = $request->getSession()->get('cursoAcademico');

        $filtrado = $request->query->all();
        //ORM::depurar($nIdAcada);

        $aListadoDocentes = [];

        if ($oUsuario) {
            /* POR AHORA NO SE VA A FILTRAR POR DPTO
            $aDatos['departamento'] = $this->orm->obtenerEntidad(null, TDepartamento::K_ENTIDAD, true);

            //se ha filtrado, obtenemos los docentes del departamento indicado        
            if (isset($filtrado['busca']) && $filtrado['busca'] == 'S') {
                $aListadoDocentes = $this->orm->obtenerPDIbyDpto($filtrado['dptoModificado']);
            } else {
                
            }*/
            $aListadoDocentes = $this->orm->obtenerPDIbyDpto(null, $nIdAcada);

            $aDatos['nombreAcada'] = $request->getSession()->get('nombreAcada');

            return $this->render(
                'asignardocente/index.html.twig',
                [
                    'datos' => $aListadoDocentes,
                    'entidades' => $aDatos,
                    'filtrado'     => $filtrado,
                    'nombre'       => $sNombre,
                    'error'        => $sError,
                    'usuarios'     => $request->getSession()->get('usuariosApp', NULL)
                ]
            );
        } else {
            //sin usuario -> al login
            return $this->redirectToRoute('ms_login', [], Response::HTTP_SEE_OTHER);
        }
    }


    /**
     * Pantalla inicial 
     * @Route("/listado/{docente}", name="asignarDocente_listado", methods={"GET"})
     */
    public function listadoindex(Request $request): Response
    {
        $idDocente = $request->get('docente');

        $aDato = [];

        //comprobamos si hay errores
        $sError = $request->getSession()->get('error', NULL);

        //inicializar el error
        $request->getSession()->set('error', NULL);

        //datos del usuario conectado
        $sNombre = $request->getSession()->get('usuarioAplicativo', NULL);
        $oUsuario = $request->getSession()->get('usuarioApp', null);

        //obtenemos el curso acad.
        $nIdAcada = $request->getSession()->get('cursoAcademico');

        $filtrado = $request->query->all();
        if ($oUsuario) {
            //si ha entrado, el admin/lectura lee todos los datos
            if (in_array('GA.ADMIN', $oUsuario->getRoles()) || in_array('GA.LECTURA', $oUsuario->getRoles())) {
                //$aDato = $this->orm->obtenerEntidad(null, $this->entidad);
                $aDato = $this->orm->obtenerTitulosCurso($nIdAcada);
                $aDatos['titulos'] = $this->orm->obtenerEntidad(null, TTitulacionMaestro::K_ENTIDAD, true);
                //$aDatos['plan'] = $this->orm->obtenerEntidad(null, TPlan::K_ENTIDAD, true);
            } elseif (in_array('GA.COORDINADOR', $oUsuario->getRoles())) {
                //si ha entrado, el coodirnador lee los datos de su facultad
                $aDatos['titulos'] = $this->orm->obtenerTitulosCoordinadorPDI($this->entidad, $oUsuario->getFacultad()->getId());
            }

            $aDatos['plan'] = [];
            $aAsignaturas = [];

            if (isset($filtrado['busca']) && $filtrado['busca'] == 'S') {
                $idTitulo = $request->get('tituloModificado', null);
                $modalidad = $request->get('modalidadModificado', null);
                $plan = $request->get('planModificado', null);
                $tipo = $request->get('tipoModificado', 1);
                $nCursoAcada = $request->getSession()->get('cursoAcademico');

                //comprobamos los valores
                if ($idTitulo && $modalidad && $plan && $nCursoAcada) {
                    $aAsignaturasGrupos =  $this->orm->obtenerAsignaturasTitulacionCurso($idTitulo, $modalidad, $plan, $nCursoAcada, $tipo);
                    $aAsignaturas =  $this->separarGrupos($aAsignaturasGrupos);
                }

                $aDatos['plan'] = $this->orm->obtenerPlanTitulacion($idTitulo, $modalidad);
            }

            $oDocente = $this->orm->obtenerEntidad($idDocente, TPersonalalta::class);
            $aDatos['creditosAsignadosDocente'] = $this->orm->sumarCreditosDocente($oDocente->getPersonal()->getId(), $nIdAcada) ?? 0;
            $aDatos['asignaturasActuales'] = $this->orm->obtenerAsignaturasPDI($idDocente, $nIdAcada);
           /* orm::depurar($oDocente->getPersonal()->getId())    ;
            orm::depurar(" $idDocente -- $nIdAcada ")    ;
            orm::depurar($aDatos['asignaturasActuales'], true)    ;*/

            $aDatos['modalidad'] = $this->orm->obtenerEntidad(null, TModalidad::K_ENTIDAD);
            $aDatos['acada'] = $nIdAcada;
            $aDatos['nombreAcada'] = $request->getSession()->get('nombreAcada');

            //ORM::depurar($aDatos['asignaturasActuales'], true); 
            $nFacultad = $oUsuario->getFacultad()->getId();
            $aFlujo = $this->orm->obtenerFlujo($nIdAcada, $nFacultad);
            $aDatos['flujo'] = is_array($aFlujo) && isset($aFlujo[0]) ? $aFlujo[0] : null;
            $aDatos['facultad'] = $nFacultad;

            return $this->render(
                'asignardocente/listadoAsignaturas.html.twig',
                [
                    'datos' => $aDato,
                    'entidades' => $aDatos,
                    'docente' => $oDocente,
                    'asignaturas' => $aAsignaturas,
                    'filtrado'     => $filtrado,
                    'nombre'       => $sNombre,
                    'error'        => $sError,
                    'usuarios'     => $request->getSession()->get('usuariosApp', NULL)
                ]
            );
        } else {
            //sin usuario -> al login
            return $this->redirectToRoute('ms_login', [], Response::HTTP_SEE_OTHER);
        }
    }

    /**
     * Pantalla inicial 
     * @Route("/comprobar/{idFacultad}", defaults={"idFacultad"=null}, name="comprobar_ratios", methods={"GET"})
     */
    public function comprobarRatios(Request $request)
    {
        //comprobamos si hay errores
        $sError = $request->getSession()->get('error', NULL);
        $sInfo = $request->getSession()->get('info', NULL);

        //inicializar el error
        $request->getSession()->set('error', NULL);
        $request->getSession()->set('info', NULL);

        //datos del usuario conectado
        $sNombre = $request->getSession()->get('usuarioAplicativo', NULL);
        $oUsuario = $request->getSession()->get('usuarioApp', null);
        $filtrado = $request->query->all();
        $avisoFacultad = null;


        if ($oUsuario) {
            //si ha entrado, el admin/lectura lee todos los datos
            if (in_array('GA.ADMIN', $oUsuario->getRoles()) || in_array('GA.LECTURA', $oUsuario->getRoles())) {
                $aDatos['titulos'] = $this->orm->obtenerEntidad(null, TTitulacionMaestro::K_ENTIDAD, true);
                $aDatos['facultad'] = $this->orm->obtenerEntidad(null, TFacultad::K_ENTIDAD, true);
                $avisoFacultad = $request->get('idFacultad', null);

                if ($avisoFacultad) {
                    $filtrado['facultadModificado'] = $avisoFacultad;
                }

                $nFacultad =  $avisoFacultad ? $avisoFacultad : $request->get('facultadModificado', null);
            } elseif (in_array('GA.COORDINADOR', $oUsuario->getRoles())) {
                //si ha entrado, el coodirnador lee los datos de su facultad
                $aDatos['titulos'] = $this->orm->obtenerTitulosCoordinadorPDI($this->entidad, $oUsuario->getFacultad()->getId());
                $nFacultad = $oUsuario->getFacultad()->getId();
            }

            $nCursoAcada = $request->getSession()->get('cursoAcademico');

            $aDatos['modalidad'] = $this->orm->obtenerEntidad(null, TModalidad::K_ENTIDAD);
            $aDatos['plan'] = [];
            $aDatos['nombreAcada'] = $request->getSession()->get('nombreAcada');

            $aFlujo = $this->orm->obtenerFlujo($nCursoAcada, $nFacultad);
            $aDatos['flujo'] = is_array($aFlujo) && isset($aFlujo[0]) ? $aFlujo[0] : null;

            $resultado = [];
            $real = [];

            //obtener los datos 
            if ((isset($filtrado['busca']) && $filtrado['busca'] == 'S')  ||  $avisoFacultad) {
                $idTitulo = $request->get('tituloModificado', null);
                $idModalidad = $request->get('modalidadModificado', null);
                $idPlan = $request->get('planModificado', null);


                $resultado['niveles'] = $this->orm->calcularTotalDoctores($nCursoAcada, $idTitulo,  $idPlan, $idModalidad, false, null, $nFacultad, $sError);
                $resultado['nivelesDoctor'] = $this->orm->calcularTotalDoctores($nCursoAcada, $idTitulo,  $idPlan, $idModalidad, true, null, $nFacultad, $sError);

                $real['niveles'] = $this->orm->calcularTotalDoctores($nCursoAcada, $idTitulo,  $idPlan, $idModalidad, false, null, $nFacultad, $sError, true);
                $real['nivelesDoctor'] = $this->orm->calcularTotalDoctores($nCursoAcada, $idTitulo,  $idPlan, $idModalidad, true, null, $nFacultad, $sError, true);


                $resultado['creditos'] = $this->orm->calcularCreditosDoctores($nCursoAcada, $idTitulo,  $idPlan, $idModalidad, false, $nFacultad, $sError);
                $resultado['creditosDoctor'] = $this->orm->calcularCreditosDoctores($nCursoAcada, $idTitulo,  $idPlan, $idModalidad, true, $nFacultad,  $sError);

                $real['creditos'] = $this->orm->calcularCreditosDoctores($nCursoAcada, $idTitulo,  $idPlan, $idModalidad, false, $nFacultad, $sError, true);
                $real['creditosDoctor'] = $this->orm->calcularCreditosDoctores($nCursoAcada, $idTitulo,  $idPlan, $idModalidad, true, $nFacultad,  $sError, true);

                $resultado['horas'] = $this->orm->calcularHorasDoctores($nCursoAcada, $idTitulo,  $idPlan, $idModalidad, false, $nFacultad,  $sError);
                $resultado['horasDoctor'] = $this->orm->calcularHorasDoctores($nCursoAcada, $idTitulo,  $idPlan, $idModalidad, true, $nFacultad, $sError);

                $real['horas'] = $this->orm->calcularHorasDoctores($nCursoAcada, $idTitulo,  $idPlan, $idModalidad, false, $nFacultad,  $sError, true);
                $real['horasDoctor'] = $this->orm->calcularHorasDoctores($nCursoAcada, $idTitulo,  $idPlan, $idModalidad, true, $nFacultad, $sError, true);

                $resultado['equivalente'] = $this->orm->calcularJornada($nCursoAcada, $idTitulo,  $idPlan, $idModalidad, $nFacultad, null,  $sError);
                $resultado['equivalenteCreditos'] = $this->orm->calcularJornada($nCursoAcada, $idTitulo,  $idPlan, $idModalidad, $nFacultad, 'C',  $sError);
                $resultado['equivalenteHoras'] = $this->orm->calcularJornada($nCursoAcada, $idTitulo,  $idPlan, $idModalidad, $nFacultad, 'H',  $sError);

                $real['equivalente'] = $this->orm->calcularJornada($nCursoAcada, $idTitulo,  $idPlan, $idModalidad, $nFacultad, null,  $sError, true);
                $real['equivalenteCreditos'] = $this->orm->calcularJornada($nCursoAcada, $idTitulo,  $idPlan, $idModalidad, $nFacultad, 'C',  $sError, true);
                $real['equivalenteHoras'] = $this->orm->calcularJornada($nCursoAcada, $idTitulo,  $idPlan, $idModalidad, $nFacultad, 'H',  $sError, true);

                $resultado['docentes'] = $this->orm->calcularDatosDocentes($nCursoAcada, $idTitulo,  $idPlan, $idModalidad, $nFacultad, $sError);

                $resultado['validada'] = $nFacultad ?  $this->orm->validadoODFacultad($nCursoAcada, $nFacultad)[0] : 0;
                $request->getSession()->set('datosPdf', $resultado['docentes']);
            }


            return $this->render(
                'asignardocente/ratios.html.twig',
                [
                    'entidades' => $aDatos,
                    'resultado' => $resultado,
                    'real' => $real,
                    'filtrado'     => $filtrado,
                    'nombre'       => $sNombre,
                    'error'        => $sError,
                    'info'        => $sInfo,
                    'usuarios'     => $request->getSession()->get('usuariosApp', NULL)
                ]
            );
        } else {
            //sin usuario -> al login
            return $this->redirectToRoute('ms_login', [], Response::HTTP_SEE_OTHER);
        }
    }

    /**
     * Pantalla inicial 
     * @Route("/comprobarPdi", name="comprobar_ratios_pdi")
     */
    public function comprobarRatiosPDI(Request $request)
    {
        //comprobamos si hay errores
        $sError = $request->getSession()->get('error', NULL);

        //inicializar el error
        $request->getSession()->set('error', NULL);

        //datos del usuario conectado
        $sNombre = $request->getSession()->get('usuarioAplicativo', NULL);
        $oUsuario = $request->getSession()->get('usuarioApp', null);
        $filtrado = $request->query->all();
        if ($oUsuario) {

            $aDatos['nombreAcada'] = $request->getSession()->get('nombreAcada');
            $nCursoAcada = $request->getSession()->get('cursoAcademico');

            $aDatos['pdiDpto'] = $this->orm->obtenerPdiDpto($nCursoAcada);
            $resultado = $this->orm->obtenerTodosDocentes($nCursoAcada);

            $nFacultad = $oUsuario->getFacultad()->getId();

            $aFlujo = $this->orm->obtenerFlujo($nCursoAcada, $nFacultad);
            $aDatos['flujo'] = is_array($aFlujo) && isset($aFlujo[0]) ? $aFlujo[0] : null;

            return $this->render(
                'asignardocente/ratiosPdi.html.twig',
                [
                    'entidades' => $aDatos,
                    'resultado' => $resultado,
                    'filtrado'     => $filtrado,
                    'nombre'       => $sNombre,
                    'error'        => $sError,
                    'usuarios'     => $request->getSession()->get('usuariosApp', NULL)
                ]
            );
        } else {
            //sin usuario -> al login
            return $this->redirectToRoute('ms_login', [], Response::HTTP_SEE_OTHER);
        }
    }


    /**
     * Para obtener las asignaturas de una titulac. 
     * @Route("/obtenerPlanesTitulo", name="obtener_planes_titulo")
     */
    public function obtenerPlanesTitulo(Request $request): Response
    {
        $idTitulo = $request->get('titulo');
        $modalidad = $request->get('modalidad', null);
        $sError       = FALSE;

        $aDatos = $this->orm->obtenerPlanTitulacion($idTitulo, $modalidad);

        if ($sError) {
            $request->getSession()->set('error', $sError);
            $this->logger->info('DDD error LOG->' . $sError);
            return new \Symfony\Component\HttpFoundation\Response($sError, \Symfony\Component\HttpFoundation\Response::HTTP_INTERNAL_SERVER_ERROR);
        } else {
            return new \Symfony\Component\HttpFoundation\Response(json_encode($aDatos), \Symfony\Component\HttpFoundation\Response::HTTP_OK);
        }
    }


    /**
     * Para obtener las asignaturas de una titulac. 
     * @Route("/obtenerTituloFacultad", name="obtener_titulo_facultad")
     */
    public function obtenerTituloFacultad(Request $request): Response
    {
        $idFacultad = $request->get('facultad');
        $sError       = FALSE;
        $aDatos = $this->orm->obtenerTitulosCoordinadorPDI(null, $idFacultad, $aSelect);

        if ($sError) {
            $request->getSession()->set('error', $sError);
            $this->logger->info('DDD error LOG->' . $sError);
            return new \Symfony\Component\HttpFoundation\Response($sError, \Symfony\Component\HttpFoundation\Response::HTTP_INTERNAL_SERVER_ERROR);
        } else {
            return new \Symfony\Component\HttpFoundation\Response(json_encode($aSelect), \Symfony\Component\HttpFoundation\Response::HTTP_OK);
        }
    }

    /**
     * Agregar o modificar 
     * @Route("/agregar", name="agregar_docente_curso", methods={"POST"})
     */
    public function agregar(Request $request, MailerInterface $mailer)
    {
        $sError       = FALSE;
        $id = $request->get('id', NULL); //para el caso de modificar
        $nAsignaturaCurso = $request->get('asignatura', null);
        $nPdi = $request->get('pdi', null);
        $nCreditos = $request->get('creditos', null);
        $nGrupo = $request->get('grupo', null);

        if ($nCreditos) {
            $this->logger->info('DDD AGREGAR-DOCENTE LOG IN ');
            $bModificar = $this->orm->modificarPdiAsignatura($id, $nAsignaturaCurso, $nPdi, $nCreditos, $nGrupo,  $sError, true);
            $this->logger->info('DDD AGREGAR-DOCENTE LOG OUT ' . $sError);
        } else {
            $aListadoAsignaturas = explode('@@', $nAsignaturaCurso);
            foreach ($aListadoAsignaturas as $value) {
                if ($value) {
                    list($codigoAsignatura, $grupo) = explode('_', $value);
                    $this->logger->info('DDD AGREGAR-DOCENTE LOG IN ');
                    $bModificar = $this->orm->modificarPdiAsignatura($id, $codigoAsignatura, $nPdi, null, $grupo,  $sError, true);
                    $this->logger->info('DDD AGREGAR-DOCENTE LOG OUT ' . $sError);
                }
            }
        }

        $this->cambiosDespuesValidar($request, $mailer);


        if ($sError || !$bModificar) {
            $request->getSession()->set('error', $sError);
            $this->logger->info('DDD error AGREGAR-DOCENTE LOG->' . $sError);
            return new \Symfony\Component\HttpFoundation\Response($sError, \Symfony\Component\HttpFoundation\Response::HTTP_INTERNAL_SERVER_ERROR);
        } else {
            return new \Symfony\Component\HttpFoundation\Response('OK', \Symfony\Component\HttpFoundation\Response::HTTP_OK);
        }
    }

    //vamos a separar cada asignatura en grupos
    private function separarGrupos($aAsignaturas)
    {
        $aSeparadas = [];
        //vamos a crear una fila por cada grupo
        foreach ($aAsignaturas as $asigna) {

            $nEsDual = count($this->orm->buscarAsignaturaDual($asigna->getId()));

            $nuevosGrupos = $nEsDual ?  $asigna->getGrupos() + 1 :  $asigna->getGrupos();

            $asigna->setEsDual($nEsDual ? $nuevosGrupos : 0);
            //si tiene varios grupos o es dual
            if ($asigna->getGrupos() > 1 || $nEsDual) {
                //recorremos todos los grupos, si es dual, incrementamos los grupos

                for ($i = 1; $i <= $nuevosGrupos; $i++) {
                    $oDato = new TAsignaturaCurso();
                    $oDato->setAsignatura($asigna->getAsignatura());
                    $oDato->setCursoAcada($asigna->getCursoAcada());
                    $oDato->setCurso($asigna->getCurso());
                    $oDato->setSemestre($asigna->getSemestre());
                    $oDato->setGrupos($i);
                    $oDato->setId($asigna->getId());
                    //en caso de ser dual, indicamos que grupo es el dual
                    $oDato->setEsDual($nEsDual ? $nuevosGrupos : 0);
                    //cambiamos el id                    
                    $aSeparadas[] = $oDato;
                }
            } else {
                $aSeparadas[] = $asigna;
            }
        }

        return $aSeparadas;
    }

    /**
     * @Route("/listadoPdiPdf" , name="listado_pdi_pdf")
     */
    public function generatePdf(Request $request): Response
    {
        $aDatosUsuario = $request->getSession()->get('usuarioApp');

        if (empty($aDatosUsuario)) {
            return $this->redirectToRoute('sin_permiso', []);
        }

        //datos para generara el documento
        $aDatos = $request->getSession()->get('datosPdf', null);

        array_unshift($aDatos['titulos'], 'Docente');
        $csv = Writer::createFromString('');
        $csv->setDelimiter(';');
        $csv->insertOne($aDatos['titulos']);
        $csv->insertAll($aDatos['filas']);
        $csv->setOutputBOM(Writer::BOM_UTF8); // Establecer la codificación UTF-8 y el BOM



        $response = new Response($csv->getContent(), 200, [
            'Content-Type' => 'text/csv',
            'Content-Disposition' => 'attachment; filename="archivo.csv"'
        ]);

        return $response;
    }

    /**
     * Para obtener las asignaturas de una titulac. 
     * @Route("/reemplazaPdi", name="reemplaza_pdi")
     */
    public function reemplazaPdi(Request $request, MailerInterface $mailer): Response
    {
        $idPdiOld = $request->get('anterior', null);
        $idPdiNew = $request->get('nuevo', null);
        $nIdAcada = $request->getSession()->get('cursoAcademico');

        $sError       = FALSE;

        $aDatos = $this->orm->sustituyePDI($idPdiOld, $idPdiNew, $nIdAcada, $sError);

        if ($sError) {
            $request->getSession()->set('error', $sError);
            $this->logger->info('DDD error LOG->' . $sError);
            return new \Symfony\Component\HttpFoundation\Response($sError, \Symfony\Component\HttpFoundation\Response::HTTP_INTERNAL_SERVER_ERROR);
        } else {
            $this->cambiosDespuesValidar($request, $mailer);

            return new \Symfony\Component\HttpFoundation\Response(json_encode($aDatos), \Symfony\Component\HttpFoundation\Response::HTTP_OK);
        }
    }

    /**
     * Comprobar si una facultad ha sido validada
     */
    private function cambiosDespuesValidar(Request $request, MailerInterface $mailer)
    {
        //obtenemos el usuario
        $oUsuario = $request->getSession()->get('usuarioApp', null);
        //el id de la facultad
        $nFacultad = $oUsuario->getFacultad()->getId();
        //el curso acad.
        $nCursoAcada = $request->getSession()->get('cursoAcademico');
        //vamos a comprobar si ya ha sido validada
        $aFlujo = $this->orm->obtenerFlujo($nCursoAcada, $nFacultad, true);
        //si existe es que la facultad ya ha sido validada, informamos de los cambios a rectorado y calidad
        if (is_array($aFlujo) && isset($aFlujo[0])) {
            $oFacultad = $this->orm->obtenerEntidad($oUsuario->getFacultad()->getId(), TFacultad::K_ENTIDAD);
            $datosMail['cuerpo'] = 'El usuario ' . $oUsuario->getNombre() . ' ha realizado una modificación en la ordenación docente de la facultad ' . $oFacultad->getNombre() . ' ';
            $datosMail['asunto'] = 'Se ha modificado la ordenación docente ya validada';
            $datosMail['facultad'] = $oFacultad->getId();
            $this->avisarRectoradoCalidad($mailer, $datosMail);
        }

        return true;
    }

    /**
     * enviar mail a calidad y rectorado
     * OJO DUPLICADA EN flujoController -- TODO unificar
     */
    private function avisarRectoradoCalidad(MailerInterface $mailer, $aDatos)
    {
        //obtenemos todos los usuarios de al app
        $oUsuarios = $this->orm->obtenerEntidad(null, Tusuarios::K_ENTIDAD);

        //los recorremos 
        foreach ($oUsuarios as $usuario) {
            //a todos los que sean de calidad o rectorado -> mail de aviso 
            if (
                $usuario->getTipo() == Tusuarios::K_TIPO_CALIDAD  ||
                $usuario->getTipo() == Tusuarios::K_TIPO_RECTOR
            ) {
                //si el usuario tiene tlf, notificamos al tlf // OJO -- TODO -- SE CADUCA LA INSTANCIA Y NO VA
                if ($usuario->getTlf()) {
                    inicioController::enviarMensajeTlf($usuario->getTlf(), $aDatos['cuerpo'], $sError);
                }
                //else {
                //en caso de no tener tlf, notificamos al mail
                //TODO -- AVISO POR MAIL HASTA TENER CLARO WHATSAPP   
                inicioController::envioMail($mailer, null, $usuario->getEmail(), $aDatos['asunto'], $aDatos['cuerpo']);
                //}
            }
        }
    }
}
