<?php

namespace App\Controller;

use App\Entity\TAsignaturaCurso;
use App\Entity\TCurso;
use App\Entity\TModalidad;
use App\Entity\TFacultad;
use App\Entity\TPlan;
use App\Entity\TDepartamento;
use App\Entity\TFlujo;
use App\Entity\TTitulacionMaestro;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;
use App\ORM\ORM;
use Psr\Log\LoggerInterface;

/**
 * @Route("/asignaturaCurso")
 */
class asignaturaCursoController extends AbstractController
{

    private $orm;
    private $logger;
    private $entidad;

    public function __construct(EntityManagerInterface $entityManager, LoggerInterface $logger)
    {
        $this->orm = new ORM($entityManager);
        $this->logger = $logger;
        $this->entidad = TAsignaturaCurso::K_ENTIDAD;
    }

    /**
     * Pantalla inicial 
     * @Route("/", name="asignatura_curso_index", methods={"GET"})
     */
    public function index(Request $request): Response
    {
        //comprobamos si hay errores
        $sError = $request->getSession()->get('error', NULL);

        //inicializar el error
        $request->getSession()->set('error', NULL);

        //datos del usuario conectado
        $sNombre = $request->getSession()->get('usuarioAplicativo', NULL);
        $oUsuario = $request->getSession()->get('usuarioApp', null);

        $filtrado = $request->query->all();
        if ($oUsuario) {

            $aDatos = $this->orm->obtenerEntidad(null, TCurso::K_ENTIDAD, true);

            return $this->render(
                'asignaturacurso/cursoAcademico.html.twig',
                [
                    'datos' => $aDatos,
                    'filtrado'     => $filtrado,
                    'nombre'       => $sNombre,
                    'error'        => $sError,
                    'usuarios'     => $request->getSession()->get('usuariosApp', NULL)
                ]
            );
        } else {
            //sin usuario -> al login
            return $this->redirectToRoute('ms_login', [], Response::HTTP_SEE_OTHER);
        }
    }

    /**
     * Pantalla inicial 
     * @Route("/listado/{idCurso}", defaults={"idCurso"=null}, name="asignatura_curso_listado", methods={"GET"})
     */
    public function listadoAsignaturas(Request $request): Response
    {
        //comprobamos si hay errores
        $sError = $request->getSession()->get('error', NULL);
        $request->getSession()->set('error', NULL);
        $oUsuario = $request->getSession()->get('usuarioApp', null);

        //obtenemos el id del curso acad.
        $nIdAcada =  $request->get('idCurso', null);

        $cursosAcademicos = $this->orm->obtenerEntidad(null, TCurso::K_ENTIDAD);
        if ($oUsuario) {
            //si tenemos lo almacenamos en SESSION        
            if ($nIdAcada) {
                $request->getSession()->set('cursoAcademico', $nIdAcada);
                //mostramos el curso seleccionado
                foreach ($cursosAcademicos as $cursos) {
                    if ($nIdAcada  == $cursos->getId()) {
                        $nCursoAcademicoDefecto = $cursos->getId();
                        $nNombreCursoAcada = $cursos->getNombre();
                    }
                }
                $request->getSession()->set('cursoAcademico', $nCursoAcademicoDefecto);
                $request->getSession()->set('nombreAcada', $nNombreCursoAcada);
            } else {
                $nIdAcada = $request->getSession()->get('cursoAcademico');
            }

            //datos del usuario conectado
            $sNombre = $request->getSession()->get('usuarioAplicativo', NULL);
            $oUsuario = $request->getSession()->get('usuarioApp', null);
            $nFacultad = $oUsuario->getFacultad()->getId();

            $filtrado = $request->query->all();
            $aDatos = [];

            //si ha entrado, el admin/lectura lee todos los datos
            if (in_array('GA.ADMIN', $oUsuario->getRoles()) || in_array('GA.LECTURA', $oUsuario->getRoles())) {
                //$aDato = $this->orm->obtenerEntidad(null, $this->entidad);
                $aDato = $this->orm->obtenerTitulosCurso($nIdAcada, $filtrado);

                $aDatos['titulos'] = $this->orm->obtenerEntidad(null, TTitulacionMaestro::K_ENTIDAD, true);
                $aDatos['asigna'] = [];
                $aDatos['cursos'] = $cursosAcademicos;
                $aDatos['facultad'] = $this->orm->obtenerEntidad(null, TFacultad::K_ENTIDAD, true);
                //estado de la facultad para el curso acad.
                $aFlujo = $this->orm->obtenerFlujo($nIdAcada, null);
            } elseif (in_array('GA.COORDINADOR', $oUsuario->getRoles())) {
                //si ha entrado, el coodirnador lee los datos de su facultad
                $aDato = $this->orm->obtenerTitulosCoordinador($this->entidad, $oUsuario->getFacultad()->getId(), $nIdAcada, $filtrado);

                $aDatos['titulos'] = $this->orm->obtenerTitulosCoordinadorPDI($this->entidad, $oUsuario->getFacultad()->getId());
                //estado de la facultad para el curso acad.
                $aFlujo = $this->orm->obtenerFlujo($nIdAcada, $nFacultad);
            }

            //carga de datos maestros para la plantilla
            $aDatos['modalidad'] = $this->orm->obtenerEntidad(null, TModalidad::K_ENTIDAD);
            $aDatos['plan'] = $this->orm->obtenerEntidad(null, TPlan::K_ENTIDAD, true);
            $aDatos['departamento'] = $this->orm->obtenerEntidad(null, TDepartamento::K_ENTIDAD, true);
            $aDatos['nombreAcada'] = $request->getSession()->get('nombreAcada');


            $aDatos['estado'] = is_array($aFlujo) && isset($aFlujo[0]) ? $aFlujo[0]->getEstado() : null;

            //comprobamos si hay avisos pendientes
            $aAvisos = [];
            if (is_array($aFlujo) && count($aFlujo) > 0) {
                foreach ($aFlujo as $flujo) {
                    if ($flujo->getEstado() == TFlujo::K_CALIDAD) {
                        //puede que ya se haya validado, procedemos a comprobar si ha sido validada
                        $aEstaValidada = $this->orm->obtenerFlujo($nIdAcada, $flujo->getFacultad()->getId(), true);
                        //pendiente del rectorado y sin validar, mostramos mensaje
                        if (empty($aEstaValidada)) {
                            $aAvisos[$flujo->getFacultad()->getId()] = $flujo->getFacultad()->getNombre();
                        }
                    }
                }
            }

            $request->getSession()->set('estadoFacultad', $aDatos['estado']);
            //ORM::depurar($aFlujo,true);  
            return $this->render(
                'asignaturacurso/index.html.twig',
                [
                    'datos' => $aDato,
                    'entidades' => $aDatos,
                    'filtrado'     => $filtrado,
                    'nombre'       => $sNombre,
                    'avisos' => $aAvisos,
                    'error'        => $sError,
                    'usuarios'     => $request->getSession()->get('usuariosApp', NULL)
                ]
            );
        } else {
            //sin usuario -> al login
            return $this->redirectToRoute('ms_login', [], Response::HTTP_SEE_OTHER);
        }
    }


    /**
     * Pantalla inicial 
     * @Route("/revisarOD", name="revisar_od", methods={"GET"})
     */
    public function reivsarOD(Request $request): Response
    {
        //comprobamos si hay errores
        $sError = $request->getSession()->get('error', NULL);
        $request->getSession()->set('error', NULL);
        $oUsuario = $request->getSession()->get('usuarioApp', null);

        //obtenemos el id del curso acad.
        $nIdAcada =  $request->getSession()->get('cursoAcademico');

        if ($oUsuario) {

            //datos del usuario conectado
            $sNombre = $request->getSession()->get('usuarioAplicativo', NULL);
            $nFacultad = $oUsuario->getFacultad()->getId();

            $filtrado = $request->query->all();
            $aDatos = [];

            if (in_array('GA.ADMIN', $oUsuario->getRoles()) || in_array('GA.LECTURA', $oUsuario->getRoles())) {
                //$aDato = $this->orm->obtenerEntidad(null, $this->entidad);
                $aDato = $this->orm->obtenerTitulosCurso($nIdAcada, $filtrado);
            } else {
                //si ha entrado, el coodirnador lee los datos de su facultad
                $aDato = $this->orm->obtenerTitulosCoordinador($this->entidad, $oUsuario->getFacultad()->getId(), $nIdAcada, $filtrado);
            }

            //para poder generar el PDF necesitamos los datos
            $request->getSession()->set('datosParaValidar_' . $oUsuario->getId(), $aDato);

            //estado de la facultad para el curso acad.
            $aFlujo = $this->orm->obtenerFlujo($nIdAcada, $nFacultad);
            $aDatos['estado'] = is_array($aFlujo) && isset($aFlujo[0]) ? $aFlujo[0]->getEstado() : null;
            $aDatos['nombreAcada'] = $request->getSession()->get('nombreAcada');

            $request->getSession()->set('estadoFacultad', $aDatos['estado']);

            return $this->render(
                'asignaturacurso/revisarOD.html.twig',
                [
                    'datos' => $aDato,
                    'entidades' => $aDatos,
                    'filtrado'     => $filtrado,
                    'nombre'       => $sNombre,
                    'error'        => $sError,
                    'usuarios'     => $request->getSession()->get('usuariosApp', NULL)
                ]
            );
        } else {
            //sin usuario -> al login
            return $this->redirectToRoute('ms_login', [], Response::HTTP_SEE_OTHER);
        }
    }

    /**
     * Agregar o modificar 
     * @Route("/agregar", name="agregar_asignatura_curso", methods={"POST"})
     */
    public function agregar(Request $request): Response
    {
        $sError       = FALSE;

        $bModificar = $this->orm->modificarasignacurso($request, $this->entidad, $sError);

        if ($sError || !$bModificar) {
            $request->getSession()->set('error', $sError);
            return new \Symfony\Component\HttpFoundation\Response($sError, \Symfony\Component\HttpFoundation\Response::HTTP_INTERNAL_SERVER_ERROR);
        } else {
            return new \Symfony\Component\HttpFoundation\Response('OK', \Symfony\Component\HttpFoundation\Response::HTTP_OK);
        }
    }

    /**
     * Para eliminar 
     * @Route("/eliminar", name="eliminar_asignatura_curso", methods={"POST"})
     */
    public function eliminar(Request $request): Response
    {
        $id = $request->get('id');
        $sError       = FALSE;

        $this->orm->eliminar($id, $this->entidad, $sError);

        if ($sError) {
            $request->getSession()->set('error', $sError);
            $this->logger->info('DDD error LOG->' . $sError);
            return new \Symfony\Component\HttpFoundation\Response($sError, \Symfony\Component\HttpFoundation\Response::HTTP_INTERNAL_SERVER_ERROR);
        } else {
            return new \Symfony\Component\HttpFoundation\Response('OK', \Symfony\Component\HttpFoundation\Response::HTTP_OK);
        }
    }

    /**
     * Para obtener las asignaturas de una titulac. 
     * @Route("/obtenerAsignaturaTitulo", name="obtener_asignatura_titulo")
     */
    public function obtenerAsignaturasTitulo(Request $request): Response
    {
        $id = $request->get('id');
        $plan =  $request->get('plan', null);
        $modalidad = $request->get('modalidad', null);
        $sError       = FALSE;

        $aDatos = $this->orm->obtenerAsignaturasTitulacion($id, $modalidad, $plan);

        if ($sError) {
            $request->getSession()->set('error', $sError);
            $this->logger->info('DDD error LOG->' . $sError);
            return new \Symfony\Component\HttpFoundation\Response($sError, \Symfony\Component\HttpFoundation\Response::HTTP_INTERNAL_SERVER_ERROR);
        } else {
            return new \Symfony\Component\HttpFoundation\Response(json_encode($aDatos), \Symfony\Component\HttpFoundation\Response::HTTP_OK);
        }
    }

    /**
     * Para obtener las asignaturas de una titulac. 
     * @Route("/obtenerDocentesDpto", name="obtener_pdi_dpto")
     */
    public function obtenerDocentesDpto(Request $request): Response
    {
        $id = $request->get('id');
        $nIdAcada = $request->getSession()->get('cursoAcademico', null);
        $sError       = FALSE;

        $aDatos = $this->orm->obtenerDocentesDpto($id, $nIdAcada);

        if ($sError) {
            $request->getSession()->set('error', $sError);
            $this->logger->info('DDD error LOG->' . $sError);
            return new \Symfony\Component\HttpFoundation\Response($sError, \Symfony\Component\HttpFoundation\Response::HTTP_INTERNAL_SERVER_ERROR);
        } else {
            return new \Symfony\Component\HttpFoundation\Response(json_encode($aDatos), \Symfony\Component\HttpFoundation\Response::HTTP_OK);
        }
    }

    /**
     * Para obtener las asignaturas de una titulac. 
     * @Route("/guardarPdiAsignatura", name="guardar_pdi_asignatura")
     */
    public function guardarPdiAsignatura(Request $request): Response
    {
        $sError       = FALSE;

        $bModificar = $this->orm->modificarPdiAsignatura($request,  $sError);

        if ($sError && $bModificar) {
            $request->getSession()->set('error', $sError);
            $this->logger->info('DDD error LOG->' . $sError);
            return new \Symfony\Component\HttpFoundation\Response($sError, \Symfony\Component\HttpFoundation\Response::HTTP_INTERNAL_SERVER_ERROR);
        } else {
            return new \Symfony\Component\HttpFoundation\Response('OK', \Symfony\Component\HttpFoundation\Response::HTTP_OK);
        }
    }
}
