<?php

namespace App\Controller;

use App\Entity\TAsignatura;
use App\Entity\TAsignaturaDual;
use App\Entity\TAsignaturaMaestro;
use App\Entity\TCurso;
use App\Entity\TModalidad;
use App\Entity\TPlan;
use App\Entity\TTitulacionMaestro;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;
use App\ORM\ORM;
use Psr\Log\LoggerInterface;

/**
 * @Route("/asignaturaDual")
 */
class asignaturaDualController extends AbstractController
{

    private $orm;
    private $logger;
    private $entidad;

    //OJO al migrar datos comprobar -- Datos por defecto para obtener las duales
    const K_Grado_infantil = 39;
    const K_modalidad_presencial = 10;
    const K_plan_2020 = 5;

    public function __construct(EntityManagerInterface $entityManager, LoggerInterface $logger)
    {
        $this->orm = new ORM($entityManager);
        $this->logger = $logger;
        $this->entidad = TAsignaturaDual::K_ENTIDAD;
    }

    /**
     * Pantalla inicial 
     * @Route("/", name="asignatura_dual_index", methods={"GET"})
     */
    public function index(Request $request): Response
    {
        //comprobamos si hay errores
        $sError = $request->getSession()->get('error', NULL);

        //inicializar el error
        $request->getSession()->set('error', NULL);

        //datos del usuario conectado
        $sNombre = $request->getSession()->get('usuarioAplicativo', NULL);
        $oUsuario = $request->getSession()->get('usuarioApp', null);

        $filtrado = $request->query->all();

        $nCurso = $request->getSession()->get('cursoAcademico');

        if ($oUsuario) {
            $aDato = $this->orm->obtenerDualCurso($nCurso);

            $aDatos['titulos'][] = $this->orm->obtenerEntidad(self::K_Grado_infantil, TTitulacionMaestro::K_ENTIDAD);
            $aPlanes = $this->orm->obtenerPlanTitulacion(self::K_Grado_infantil);
            $aDatos['planes'] = $aPlanes;

            //ORM::depurar($aDatos['titulos'], true);

            $aDatos['modalidad'] = $this->orm->obtenerEntidad(null, TModalidad::K_ENTIDAD);
            $aDatos['asignaturas'] = $this->obtenerAsignaturasDuales(self::K_modalidad_presencial,  self::K_plan_2020, $nCurso);
            $aDatos['cursos'] = $nCurso;
            $aDatos['nombreAcada'] = $request->getSession()->get('nombreAcada');

            return $this->render(
                'asignaturadual/index.html.twig',
                [
                    'datos' => $aDato,
                    'entidades' => $aDatos,
                    'filtrado'     => $filtrado,
                    'nombre'       => $sNombre,
                    'error'        => $sError,
                    'usuarios'     => $request->getSession()->get('usuariosApp', NULL)
                ]
            );
        } else {
            //sin usuario -> al login
            return $this->redirectToRoute('ms_login', [], Response::HTTP_SEE_OTHER);
        }
    }

    /**
     * Agregar o modificar 
     * @Route("/agregar", name="agregar_asignatura_dual", methods={"POST"})
     */
    public function agregar(Request $request): Response
    {
        $sError       = FALSE;
        $nCurso = $request->getSession()->get('cursoAcademico');
        $bModificar = $this->orm->crearAsignaturaDual($request, self::K_Grado_infantil, $nCurso, $sError);

        if ($sError || !$bModificar) {
            $request->getSession()->set('error', $sError);
            return new \Symfony\Component\HttpFoundation\Response($sError, \Symfony\Component\HttpFoundation\Response::HTTP_INTERNAL_SERVER_ERROR);
        } else {
            return new \Symfony\Component\HttpFoundation\Response('OK', \Symfony\Component\HttpFoundation\Response::HTTP_OK);
        }
    }

    /**
     * Para eliminar 
     * @Route("/eliminar", name="eliminar_asignatura_dual", methods={"POST"})
     */
    public function eliminar(Request $request): Response
    {
        $id = $request->get('id');
        $sError       = FALSE;

        $this->orm->eliminar($id, $this->entidad, $sError);

        if ($sError) {
            $request->getSession()->set('error', $sError);
            $this->logger->info('DDD error LOG->' . $sError);
            return new \Symfony\Component\HttpFoundation\Response($sError, \Symfony\Component\HttpFoundation\Response::HTTP_INTERNAL_SERVER_ERROR);
        } else {
            return new \Symfony\Component\HttpFoundation\Response('OK', \Symfony\Component\HttpFoundation\Response::HTTP_OK);
        }
    }


    /**
     * Para obtener las asignaturas de una titulac. 
     * @Route("/obtenerAsignaturaTitulo", name="obtener_asignatura_titulo_curso")
     */
    public function obtenerAsignaturasTitulo(Request $request): Response
    {
        $id = $request->get('id');
        $plan =  $request->get('plan', null);
        $modalidad = $request->get('modalidad', null);
        $curso = $request->get('curso', null);
        $sError       = FALSE;

        $aAsignaturas = $this->obtenerAsignaturasDuales($modalidad, $plan, $curso);

        if ($sError) {
            $request->getSession()->set('error', $sError);
            $this->logger->info('DDD error LOG->' . $sError);
            return new \Symfony\Component\HttpFoundation\Response($sError, \Symfony\Component\HttpFoundation\Response::HTTP_INTERNAL_SERVER_ERROR);
        } else {
            return new \Symfony\Component\HttpFoundation\Response(json_encode($aAsignaturas), \Symfony\Component\HttpFoundation\Response::HTTP_OK);
        }
    }


    private function obtenerAsignaturasDuales($modalidad, $plan, $curso)
    {
        //obtenemos los datos filtrados
        $aDatos = $this->orm->obtenerAsignaturasTitulacionCurso(self::K_Grado_infantil, $modalidad, $plan, $curso, TAsignatura::K_TIPO_NORMAL);

        //los pasamos a array para poder devolverlos
        $aAsignaturas = [];
        foreach ($aDatos as $value) {
            $asignatura['id'] = $value->getId();
            $asignatura['nombre'] = $value->getAsignatura()->getAsignatura()->getNombre();
            //comprobamos que solamente se muestren las que son duales
            $nEsDual = $this->orm->comprobarasignaturadual($value->getAsignatura()->getAsignatura()->getId(), $sError);
            if ($nEsDual > 0) {
                $aAsignaturas[] = $asignatura;
            }
        }

        usort($aAsignaturas, function ($a, $b) {
            return strcmp($a['nombre'], $b['nombre']);
        });

        return $aAsignaturas;
    }
}
