<?php

namespace App\Controller;

use App\Entity\TAsignaturaCurso;
use App\Entity\TAsignaturaPdi;
use App\Entity\TDepartamento;
use App\Entity\TPlan;
use App\Entity\TModalidad;
use App\Entity\TDptoPdi;
use App\Entity\TTitulacion;
use App\Entity\TTitulacionMaestro;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;
use App\ORM\ORM;
use Psr\Log\LoggerInterface;

/**
 * @Route("/asignaturapdi")
 */
class asignaturaPdiController extends AbstractController
{

    private $orm;
    private $logger;
    private $entidad;

    public function __construct(EntityManagerInterface $entityManager, LoggerInterface $logger)
    {
        $this->orm = new ORM($entityManager);
        $this->logger = $logger;
        $this->entidad = TAsignaturaPdi::K_ENTIDAD;
    }

    /**
     * Pantalla inicial 
     * @Route("/", name="asignatura_pdi_index", methods={"GET"})
     */
    public function index(Request $request): Response
    {
        //comprobamos si hay errores
        $sError = $request->getSession()->get('error', NULL);

        //inicializar el error
        $request->getSession()->set('error', NULL);

        //datos del usuario conectado
        $sNombre = $request->getSession()->get('usuarioAplicativo', NULL);
        $oUsuario = $request->getSession()->get('usuarioApp', null);
        $nIdAcada = $request->getSession()->get('cursoAcademico');

        $filtrado = $request->query->all();
        if ($oUsuario) {

            //si ha entrado, el admin lee todos los datos
            if (in_array('GA.ADMIN', $oUsuario->getRoles())  || in_array('GA.LECTURA', $oUsuario->getRoles())) {
                $aDato['titulaciones'] = $this->orm->obtenerEntidad(null, TTitulacionMaestro::K_ENTIDAD, true);
                $aDatos = $this->orm->obtenerPDICurso($nIdAcada);
            } elseif (in_array('GA.COORDINADOR', $oUsuario->getRoles())) {
                //si ha entrado, el coodirnador lee los datos de su facultad
                $aDato['titulaciones'] = $this->orm->obtenerTitulosCoordinadorPDI(TTitulacion::K_ENTIDAD, $oUsuario->getFacultad()->getId());
                $aDatos = $this->orm->obtenerPDICurso($nIdAcada, $oUsuario->getFacultad()->getId());
            }



            $aDato['departamento'] = $this->orm->obtenerEntidad(null, TDepartamento::K_ENTIDAD, true);
            $aDato['modalidad'] = $this->orm->obtenerEntidad(null, TModalidad::K_ENTIDAD);
            $aDato['plan'] = $this->orm->obtenerEntidad(null, TPlan::K_ENTIDAD, true);
            $aDato['nombreAcada'] = $request->getSession()->get('nombreAcada');
            //$aDato['pdiDpto'] = $this->orm->obtenerPdiDpto($nIdAcada);

            $aDato['estado'] = $request->getSession()->get('estadoFacultad', null);

            return $this->render(
                'asignaturapdi/index.html.twig',
                [
                    'datos' => $aDatos,
                    'entidades' => $aDato,
                    'filtrado'     => $filtrado,
                    'nombre'       => $sNombre,
                    'error'        => $sError,
                    'usuarios'     => $request->getSession()->get('usuariosApp', NULL)
                ]
            );
        } else {
            //sin usuario -> al login
            return $this->redirectToRoute('ms_login', [], Response::HTTP_SEE_OTHER);
        }
    }

    /**
     * Agregar o modificar 
     * @Route("/agregar", name="agregar_asignatura_pdi", methods={"POST"})
     */
    public function agregar(Request $request): Response
    {
        $sError       = FALSE;

        $bModificar = $this->orm->modificar($request, $this->entidad, $sError);

        if ($sError || !$bModificar) {
            $request->getSession()->set('error', $sError);
            return new \Symfony\Component\HttpFoundation\Response($sError, \Symfony\Component\HttpFoundation\Response::HTTP_INTERNAL_SERVER_ERROR);
        } else {
            return new \Symfony\Component\HttpFoundation\Response('OK', \Symfony\Component\HttpFoundation\Response::HTTP_OK);
        }
    }

    /**
     * Para eliminar 
     * @Route("/eliminar", name="eliminar_asignatura_pdi", methods={"POST"})
     */
    public function eliminar(Request $request): Response
    {
        $id = $request->get('id');
        $sError       = FALSE;

        //TODO -- SI SE USA MUCHO -> ES MEJORABLE

        //obtenemos la entidad
        $oAsignaturaPDI = $this->orm->obtenerEntidad($id, $this->entidad);
        //obtenemos el id
        $idAsignatura = $oAsignaturaPDI->getAsignatura()->getId();
        //comprobamos si es DUAL
        $nEsDual = $this->orm->obtenerSiEsAsignaturaDual($idAsignatura, $sError);

        //TODO -- Avisar en caso de modificar una vez validada la convocatoria

        if ($nEsDual) {
            //en caso de ser dual, eliminamos la dual
            $oDual = $this->orm->localizarAsignaturaPDI($nEsDual, $oAsignaturaPDI->getPdi()->getId(), $oAsignaturaPDI->getCursoAcada()->getId(), $oAsignaturaPDI->getGrupo());
            if ($oDual){
                $this->orm->eliminar($oDual->getId(), $this->entidad, $sError);
            }
        }

        $this->orm->eliminar($id, $this->entidad, $sError);

        if ($sError) {
            $request->getSession()->set('error', $sError);
            $this->logger->info('DDD error LOG->' . $sError);
            return new \Symfony\Component\HttpFoundation\Response($sError, \Symfony\Component\HttpFoundation\Response::HTTP_INTERNAL_SERVER_ERROR);
        } else {
            return new \Symfony\Component\HttpFoundation\Response('OK', \Symfony\Component\HttpFoundation\Response::HTTP_OK);
        }
    }
}
