<?php

namespace App\Controller;

use App\Entity\TFacultad;
use App\Entity\TFlujo;
use App\Entity\Tusuarios;
use Doctrine\ORM\EntityManagerInterface;

use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;
use App\ORM\ORM;
use Psr\Log\LoggerInterface;
use Symfony\Component\Mailer\MailerInterface;

use Dompdf\Dompdf;
use Dompdf\Options;

/**
 * @Route("/flujo")
 */
class flujoController extends AbstractController
{

    private $orm;
    private $logger;
    private $entidad;

    public function __construct(EntityManagerInterface $entityManager, LoggerInterface $logger)
    {
        $this->orm = new ORM($entityManager);
        $this->logger = $logger;
        $this->entidad = TFlujo::K_ENTIDAD;
    }


    /**
     * Pantalla inicial 
     * @Route("/", name="flujo_index", methods={"GET"})
     */
    public function index(Request $request): Response
    {
        //comprobamos si hay errores
        $sError = $request->getSession()->get('error', NULL);

        //inicializar el error
        $request->getSession()->set('error', NULL);

        //datos del usuario conectado
        $sNombre = $request->getSession()->get('usuarioAplicativo', NULL);
        $oUsuario = $request->getSession()->get('usuarioApp', null);

        $filtrado = $request->query->all();

        $aDatos['nombreAcada'] = $request->getSession()->get('nombreAcada');

        if ($oUsuario) {
            //si ha entrado, procedemos a cargar lo valores necesarios

            $aDatos['estados'] = $this->orm->obtenerEntidad(null, $this->entidad);

            return $this->render(
                'flujo/index.html.twig',
                [
                    'datos' => $aDatos,
                    'filtrado'     => $filtrado,
                    'nombre'       => $sNombre,
                    'error'        => $sError,
                    'usuarios'     => $request->getSession()->get('usuariosApp', NULL)
                ]
            );
        }
        //sin usuario o sin permisos-> al login
        return $this->redirectToRoute('ms_login', [], Response::HTTP_SEE_OTHER);
    }



    /**
     * Enviar la orde. docente de una facultad 
     * @Route("/enviarOD", name="enviar_ordenacion", methods={"POST"})
     */
    public function coordinadorEnviaOD(Request $request, MailerInterface $mailer): Response
    {

        $sError       = FALSE;

        $oUsuario = $request->getSession()->get('usuarioApp', null);
        $nIdAcada = $request->getSession()->get('cursoAcademico');
        $filtrado = $request->request->all();

        $bEnviaCoordinador = isset($filtrado['enviaCoordinador']) ? true : false;
        $bEnviaCalidad = isset($filtrado['enviaCalidad']) ? true : false;
        $bEnviaRector = isset($filtrado['enviaRector']) ? true : false;


        $bRechazar = isset($filtrado['rechazar']) ? true : false;
        $sMotivo = isset($filtrado['motivo']) ? $filtrado['motivo'] :  null;
        $sPropuesta = isset($filtrado['propuesta']) ? $filtrado['propuesta'] :  null;
        $aParam = [];

        if ($oUsuario) {
            $oUsuarioBD = $this->orm->obtenerEntidad($oUsuario->getId(), Tusuarios::K_ENTIDAD);

            //si es calidad/rector ha filtrado por facultad, el resto obtenemos la facultad a la que pertenece
            $nFacultad = isset($filtrado['facultad']) ? $filtrado['facultad'] :  $oUsuarioBD->getFacultad()->getId();
            $oFacultad = $this->orm->obtenerEntidad($nFacultad, TFacultad::K_ENTIDAD);

            $sNombre = $oUsuarioBD->getNombre();
            //ORM::depurar($filtrado, true);
            if ($bEnviaCoordinador) {
                $sTexto  = $sNombre . ' ha enviado a decanato la ordenación académica de la facultad de ' . $oFacultad->getNombre();
            } elseif ($bEnviaCalidad) {
                $sTexto  = $sNombre . ' ha enviado a rectorado/calidad la ordenación académica de la facultad de ' . $oFacultad->getNombre();
                //al enviar calidad, tenemos que generar el PDF con la O.D. que ha validado 
                $docentes = $this->orm->calcularDatosDocentes($nIdAcada, null, null, null, $nFacultad, $sError);
                //guardamos el fichero para almacenarlo en la BD
                $aParam['fichero'] = $this->generarPDF($docentes, $oUsuario);
            } elseif ($bEnviaRector) {
                //el rector valida por facultades
                $sTexto  = $sNombre . ' ha validado la ordenación docente de la facultad de ' . $oFacultad->getNombre();
            } elseif ($bRechazar) {
                //el rector rechaza por facultades
                $sTexto  = $sNombre . ' ha rechazado la ordenación docente de la facultad de ' . $oFacultad->getNombre() . ' Motivo: ' . $sMotivo . ' Propuesta: ' . $sPropuesta;
            }

            $aParam['acada'] = $nIdAcada;
            $aParam['coordinador'] = $bEnviaCoordinador;
            $aParam['calidad'] = $bEnviaCalidad;
            $aParam['rector'] = $bEnviaRector;
            $aParam['rechazo'] = $bRechazar;
            $aParam['texto'] = $sTexto;
            $aParam['facultad'] = $nFacultad;

            $bModificar = $this->orm->modificarflujo($oUsuarioBD, $aParam, $sError);

            if ($sError || !$bModificar) {
                $request->getSession()->set('error', $sError);
                return new \Symfony\Component\HttpFoundation\Response($sError, \Symfony\Component\HttpFoundation\Response::HTTP_INTERNAL_SERVER_ERROR);
            } else {
                //sin errores, se notifica al responsable de la facultad            

                //el coordinador avisa al decanato 
                if ($bEnviaCoordinador) {
                    $mail = $oFacultad->getMail();
                    inicioController::envioMail($mailer, null, $mail, 'Envío de ordenación docente', $sTexto);
                    $request->getSession()->set('info', 'Se ha validado la ordenación docente');
                } elseif ($bEnviaCalidad) {
                    $datosMail['cuerpo'] = $sTexto;
                    $datosMail['asunto'] = 'Se ha validado la ordenación docente';
                    $datosMail['facultad'] = $nFacultad;
                    $this->avisarRectoradoCalidad($mailer, $datosMail);
                    $request->getSession()->set('info', 'Se ha validado la ordenación docente');
                } elseif ($bEnviaRector) {
                    //notificamos a calidad/rectorado que se ha validado la OD
                    $datosMail['cuerpo'] = $sTexto;
                    $datosMail['asunto'] = 'Se ha validado la ordenación docente';
                    $datosMail['facultad'] = $nFacultad;
                    $this->avisarRectoradoCalidad($mailer, $datosMail);
                    //se avisa al decanato
                    $mail = $oFacultad->getMail();
                    inicioController::envioMail($mailer, null, $mail, $datosMail['asunto'], $datosMail['cuerpo']);
                    $request->getSession()->set('info', 'Se ha validado la ordenación docente');
                } elseif ($bRechazar) {
                    $datosMail['cuerpo'] = $sTexto;
                    $datosMail['asunto'] = 'Se ha rechazado la ordenación docente';
                    $datosMail['rechazar'] = true;
                    $datosMail['facultad'] = $nFacultad;
                    //$mail = $oFacultad->getMail();                    
                    //inicioController::envioMail($mailer, null, $mail, 'Rechazo de ordenación docente', $sTexto);
                    $this->avisarRechazoCalidad($mailer, $datosMail);
                    $request->getSession()->set('info', 'Se ha rechazado la ordenación docente');
                }
                return new \Symfony\Component\HttpFoundation\Response('OK', \Symfony\Component\HttpFoundation\Response::HTTP_OK);
            }
        }

        //sin usuario o sin permisos-> al login
        return $this->redirectToRoute('ms_login', [], Response::HTTP_SEE_OTHER);
    }

    /**
     * @Route("/asignaturasValidadas" , name="listado_asignaturas_validadas_pdf")
     */
    public function asignaturasValidadasPdf(Request $request, MailerInterface $mailer): Response
    {
        $nTipo = $request->get('tipo', null);
        $esFinal = $request->get('final', null);
        $bRechazar = $request->get('rechazar', false);
        $sMotivo = $request->get('motivo', false);
        $sPropuesta = $request->get('propuesta', false);
        $sError = null;

        $oUsuario = $request->getSession()->get('usuarioApp', null);
        $nIdAcada = $request->getSession()->get('cursoAcademico');
        $filtrado = $request->request->all();


        if (empty($oUsuario)) {
            return $this->redirectToRoute('sin_permiso', []);
        }

        $bEnviaCoordinador = false;
        $oUsuarioBD = $this->orm->obtenerEntidad($oUsuario->getId(), Tusuarios::K_ENTIDAD);

        $oFacultad = $this->orm->obtenerEntidad($oUsuarioBD->getFacultad()->getId(), TFacultad::K_ENTIDAD);


        if ($bRechazar) {
            $sTexto  = $oUsuarioBD->getNombre() . ' ha rechazado la ordenación académica de la facultad de ' . $oFacultad->getNombre() . ' Motivo: ' . $sMotivo . ' Propuesta: ' . $sPropuesta;
        } else {
            $sTexto  = $oUsuarioBD->getNombre() . ' ha validado la ordenación académica de la facultad de ' . $oFacultad->getNombre();
        }

        $aParam = [];
        $aParam['acada'] = $nIdAcada;
        $aParam['coordinador'] = $bEnviaCoordinador;
        $aParam['calidad'] = '';
        $aParam['texto'] = $sTexto;
        $aParam['tipo'] = $nTipo;
        $aParam['final'] = $esFinal;
        $aParam['rechazo'] = $bRechazar;

        //si es un rechazo
        if ($bRechazar) {
            $bModificar = $this->orm->modificarflujo($oUsuarioBD, $aParam, $sError);
            if ($sError || !$bModificar) {
                $request->getSession()->set('error', $sError);
                return new \Symfony\Component\HttpFoundation\Response($sError, \Symfony\Component\HttpFoundation\Response::HTTP_INTERNAL_SERVER_ERROR);
            } else {

                $datosMail['cuerpo'] = $sTexto;
                $datosMail['asunto'] = 'Se ha rechazado la ordenación docente';
                $datosMail['rechazar'] = true;
                $datosMail['facultad'] = $oFacultad->getId();
                //se notifica el rechazo    
                $mail = $oFacultad->getMail();
                inicioController::envioMail($mailer, null, $mail, 'Rechazo de ordenación docente', $sTexto);
                $this->avisarRectoradoCalidad($mailer, $datosMail);

                return new \Symfony\Component\HttpFoundation\Response('OK', \Symfony\Component\HttpFoundation\Response::HTTP_OK);
            }
        } elseif ($nTipo) {
            //vamos a validar las  asignaturas con grupos > 1 o las asignaturas que no se imparten
            //datos para generar el documento
            $aDatos = $request->getSession()->get('datosParaValidar_' . $oUsuarioBD->getId(), null);
            $aParam['fichero'] = $this->generarPDF($aDatos, $oUsuario, $nTipo);
            $bModificar = $this->orm->modificarflujo($oUsuarioBD, $aParam, $sError);

            if ($sError || !$bModificar) {
                $request->getSession()->set('error', $sError);
                return new \Symfony\Component\HttpFoundation\Response($sError, \Symfony\Component\HttpFoundation\Response::HTTP_INTERNAL_SERVER_ERROR);
            } else {
                return new \Symfony\Component\HttpFoundation\Response('OK', \Symfony\Component\HttpFoundation\Response::HTTP_OK);
            }
        } else {
            $request->getSession()->set('error', $sError);
            return new \Symfony\Component\HttpFoundation\Response($sError, \Symfony\Component\HttpFoundation\Response::HTTP_INTERNAL_SERVER_ERROR);
        }
    }



    /**
     * enviar mail a calidad y rectorado
     */
    private function avisarRectoradoCalidad(MailerInterface $mailer, $aDatos)
    {
        //obtenemos todos los usuarios de al app
        $oUsuarios = $this->orm->obtenerEntidad(null, Tusuarios::K_ENTIDAD);

        //los recorremos 
        foreach ($oUsuarios as $usuario) {
            //a todos los que sean de calidad o rectorado -> mail de aviso // al rechazar no notificamos a rectorado
            if (($usuario->getTipo() == Tusuarios::K_TIPO_CALIDAD && $usuario->getFacultad()->getId() == $aDatos['facultad']) ||
                ($usuario->getTipo() == Tusuarios::K_TIPO_COORDINADOR && $usuario->getFacultad()->getId() == $aDatos['facultad'] && isset($aDatos['rechazar'])) ||
                ($usuario->getTipo() == Tusuarios::K_TIPO_RECTOR && !isset($aDatos['rechazar']))
            ) {
                //si el usuario tiene tlf, notificamos al tlf
                if ($usuario->getTlf()) {
                    inicioController::enviarMensajeTlf($usuario->getTlf(), $aDatos['cuerpo'], $sError);
                } else {
                    //en caso de no tener tlf, notificamos al mail
                    inicioController::envioMail($mailer, null, $usuario->getEmail(), $aDatos['asunto'], $aDatos['cuerpo']);
                }
            }
        }
    }


    /**
     * enviar mail a decanato y coordinadores en caso de rechazo
     */
    private function avisarRechazoCalidad(MailerInterface $mailer, $aDatos)
    {
        //obtenemos todos los usuarios de al app
        $oUsuarios = $this->orm->obtenerEntidad(null, Tusuarios::K_ENTIDAD);

        //los recorremos 
        foreach ($oUsuarios as $usuario) {
            //a decanato y coordinadores -> mail de aviso al rechazar no notificamos a rectorado
            if (
                ($usuario->getTipo() == Tusuarios::K_TIPO_DECANO && $usuario->getFacultad()->getId() == $aDatos['facultad']) ||
                ($usuario->getTipo() == Tusuarios::K_TIPO_COORDINADOR && $usuario->getFacultad()->getId() == $aDatos['facultad'])
            ) {

                //si el usuario tiene tlf, notificamos al tlf
                if ($usuario->getTlf()) {
                    inicioController::enviarMensajeTlf($usuario->getTlf(), $aDatos['cuerpo'], $sError);
                } else {
                    //en caso de no tener tlf, notificamos al mail
                    inicioController::envioMail($mailer, null, $usuario->getEmail(), $aDatos['asunto'], $aDatos['cuerpo']);
                }
            }
        }
    }

    /**
     * @Route("/asignaturasValidadasPDF/{id}" , name="asignaturas_validadas_pdf")
     */
    public function descargarArchivoAction(Request $request)
    {
        $id = $request->get('id', null);
        $entidad = $this->orm->obtenerEntidad($id, TFlujo::K_ENTIDAD);

        // Obtener el contenido del atributo blob de la entidad
        $contenidoBlob = stream_get_contents($entidad->getArchivo());

        // Crear una respuesta de tipo Response para la descarga del archivo
        $response = new Response($contenidoBlob);
        $response->headers->set('Content-Type', 'application/pdf'); // Establecer el tipo de contenido adecuado
        $response->headers->set('Content-Disposition', 'attachment; filename="DocValidaDecanato.pdf"'); // Nombre del archivo para la descarga

        return $response;
    }

    /**
     * genera un PDF para almacenar en la BD
     */
    private function generarPDF($aDatos, $oUsuario, $nTipo = null)
    {
        // Opciones de Dompdf (opcional)
        $options = new Options();
        $options->set('orientation', 'landscape');
        $options->set('isHtml5ParserEnabled', true);
        $options->set('isRemoteEnabled', true);
        $options->set('isPhpEnabled', true);

        // Crea una instancia de Dompdf
        $dompdf = new Dompdf();
        $dompdf->setOptions($options);
        $dompdf->set_paper('letter', 'landscape');

        // Renderiza el HTML a PDF
        if ($nTipo) {
            $html = $this->renderView('asignardocente/tablaRatiosPdiPdf.html.twig', [
                'datos' =>  $aDatos,
                'tipo' => $nTipo,
                'usuario' => $oUsuario,
            ]);
        } else {
            $html = $this->renderView('asignardocente/ordenacionDocentePdf.html.twig', [
                'datos' =>  $aDatos,
                'usuario' => $oUsuario,
            ]);
        }

        $dompdf->loadHtml($html);
        // Renderiza el PDF
        $dompdf->render();

        //por si queremos ver el PDF 
        //return new Response($dompdf->output(), 200, ['Content-Type' => 'application/pdf',]);

        // Envía el PDF generado como respuesta            
        return $dompdf->output();
    }
}
