<?php

namespace App\Controller;

use App\Entity\TFacultad;
use App\Entity\TModalidad;
use App\Entity\TPlan;
use App\Entity\TTitulacion;
use App\Entity\TTitulacionMaestro;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;
use App\ORM\ORM;
use Psr\Log\LoggerInterface;

/**
 * @Route("/titulacion")
 */
class titulacionController extends AbstractController
{

    private $orm;
    private $logger;
    private $entidad;

    public function __construct(EntityManagerInterface $entityManager, LoggerInterface $logger)
    {
        $this->orm = new ORM($entityManager);
        $this->logger = $logger;
        $this->entidad = TTitulacion::K_ENTIDAD;
    }

    /**
     * Pantalla inicial 
     * @Route("/", name="titulacion_index", methods={"GET"})
     */
    public function index(Request $request): Response
    {
        //comprobamos si hay errores
        $sError = $request->getSession()->get('error', NULL);

        //inicializar el error
        $request->getSession()->set('error', NULL);

        //datos del usuario conectado
        $sNombre = $request->getSession()->get('usuarioAplicativo', NULL);
        $oUsuario = $request->getSession()->get('usuarioApp', null);

        $filtrado = $request->query->all();

        if ($oUsuario) {
            //si ha entrado, procedemos a cargar lo valores necesarios
            if (in_array('GA.ADMIN', $oUsuario->getRoles()) || in_array('GA.LECTURA', $oUsuario->getRoles())) {

                $aDato = $this->orm->obtenerEntidad(null, $this->entidad);
                $aDatos['plan'] = $this->orm->obtenerEntidad(null, TPlan::K_ENTIDAD, true);
                $aDatos['titulo'] = $this->orm->obtenerEntidad(null, TTitulacionMaestro::K_ENTIDAD, true);
                $aDatos['modalidad'] = $this->orm->obtenerEntidad(null, TModalidad::K_ENTIDAD);
                $aDatos['facultad'] = $this->orm->obtenerEntidad(null, TFacultad::K_ENTIDAD, true);

                return $this->render(
                    'titulacion/index.html.twig',
                    [
                        'datos' => $aDato,
                        'filtrado'     => $filtrado,
                        'entidades' => $aDatos,
                        'nombre'       => $sNombre,
                        'error'        => $sError,
                        'usuarios'     => $request->getSession()->get('usuariosApp', NULL)
                    ]
                );
            }
        }

        //sin usuario -> al login
        return $this->redirectToRoute('ms_login', [], Response::HTTP_SEE_OTHER);
    }

    /**
     * Agregar o modificar 
     * @Route("/agregar", name="agregar_titulacion", methods={"POST"})
     */
    public function agregar(Request $request): Response
    {
        $sError       = FALSE;

        $bModificar = $this->orm->modificartitulacion($request, $this->entidad, $sError);

        if ($sError || !$bModificar) {
            $request->getSession()->set('error', $sError);
            return new \Symfony\Component\HttpFoundation\Response($sError, \Symfony\Component\HttpFoundation\Response::HTTP_INTERNAL_SERVER_ERROR);
        } else {
            return new \Symfony\Component\HttpFoundation\Response('OK', \Symfony\Component\HttpFoundation\Response::HTTP_OK);
        }
    }

    /**
     * Para eliminar 
     * @Route("/eliminar", name="eliminar_titulacion", methods={"POST"})
     */
    public function eliminar(Request $request): Response
    {
        $id = $request->get('id');
        $sError       = FALSE;

        $this->orm->eliminar($id, $this->entidad, $sError);

        if ($sError) {
            $request->getSession()->set('error', $sError);
            $this->logger->info('DDD error LOG->' . $sError);
            return new \Symfony\Component\HttpFoundation\Response($sError, \Symfony\Component\HttpFoundation\Response::HTTP_INTERNAL_SERVER_ERROR);
        } else {
            return new \Symfony\Component\HttpFoundation\Response('OK', \Symfony\Component\HttpFoundation\Response::HTTP_OK);
        }
    }
}
