<?php

namespace App\Controller;

use App\Entity\TFacultad;
use App\Entity\Tusuarios;

use Doctrine\ORM\EntityManagerInterface;

use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;
use Psr\Log\LoggerInterface;

use App\ORM\ORM;

/**
 * @Route("/usuarios")
 */
class usuariosController extends AbstractController
{

    private $orm;
    private $logger;
    private $entidad;


    public function __construct(EntityManagerInterface $entityManager, LoggerInterface $logger)
    {
        $this->orm = new ORM($entityManager);
        $this->logger = $logger;
        $this->entidad = Tusuarios::K_ENTIDAD;
    }

    /**
     * Pantalla inicial para las peticiones.
     * @Route("/", name="usuarios_index", methods={"GET"})
     */
    public function index(Request $request, EntityManagerInterface $entityManager): Response
    {
        //comprobamos si hay errores
        $sError = $request->getSession()->get('error', NULL);

        //inicializar el error
        $request->getSession()->set('error', NULL);

        //datos del usuario conectado
        $sNombre = $request->getSession()->get('usuarioAplicativo', NULL);

        $oUsuario = $request->getSession()->get('usuarioApp', null);

        if (empty($oUsuario)) {
            return $this->redirectToRoute('ms_login');
        }

        $filtrado = $request->query->all();

        //si ha entrado, procedemos a cargar lo valores necesarios
        if (in_array('GA.ADMIN', $oUsuario->getRoles()) || in_array('GA.LECTURA', $oUsuario->getRoles()) ) {
            
            $aDatos = $this->orm->obtenerUsuarios();

            $aEntidades['facultad'] = $this->orm->obtenerEntidad(null, TFacultad::K_ENTIDAD);

            return $this->render(
                'usuarios/index.html.twig',
                [
                    'datos' => $aDatos,
                    'filtrado'     => $filtrado,
                    'entidades' => $aEntidades,
                    'nombre'       => $sNombre,
                    'error'        => $sError,
                    'usuarios'     => $request->getSession()->get('usuariosApp', NULL)
                ]
            );
        } else {
            //no se encuentra en session, redirigimos al login
            return $this->redirectToRoute('sin_permiso', [], Response::HTTP_SEE_OTHER);
        }
    }

    /**
     * Agregar o modificar peticiones
     * @Route("/agregar", name="agregar_usuario", methods={"POST"})
     */
    public function agregar(Request $request, EntityManagerInterface $entityManager): Response
    {
        $sError       = FALSE;

        $bModificar = $this->orm->modificarUsuarios($request, $sError);

        if ($sError || !$bModificar) {
            $request->getSession()->set('error', $sError);
            return new \Symfony\Component\HttpFoundation\Response($sError, \Symfony\Component\HttpFoundation\Response::HTTP_INTERNAL_SERVER_ERROR);
        } else {
            return new \Symfony\Component\HttpFoundation\Response('OK', \Symfony\Component\HttpFoundation\Response::HTTP_OK);
        }
    }

    /**
     * Para eliminar una petici.
     * @Route("/eliminar", name="eliminar_usuario", methods={"POST"})
     */
    public function eliminar(Request $request, EntityManagerInterface $entityManager): Response
    {
        $id = $request->get('id');
        $sError       = FALSE;

        $this->orm->eliminar($id, $this->entidad, $sError);

        if ($sError) {
            $request->getSession()->set('error', $sError);
            return new \Symfony\Component\HttpFoundation\Response($sError, \Symfony\Component\HttpFoundation\Response::HTTP_INTERNAL_SERVER_ERROR);
        } else {
            return new \Symfony\Component\HttpFoundation\Response('OK', \Symfony\Component\HttpFoundation\Response::HTTP_OK);
        }
    }
}
