<?php

namespace App\Entity\GP;

use Doctrine\ORM\Mapping as ORM;

/**
 * TPersonalalta
 *
 * @ORM\Table(name="personal_alta_t", schema="unam_gp")
 * @ORM\Entity
 */
class TPersonalalta
{
    const K_ENTIDAD = 'App\Entity\GP\TPersonalalta';

    /**
     * @var int
     *
     * @ORM\Column(name="id", type="integer", nullable=false)
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="SEQUENCE")
     * @ORM\SequenceGenerator(sequenceName="unam_gp.personal_alta_t_id_seq", allocationSize=1, initialValue=1)
     */
    private $id;

    /**
     * @var \TContrato
     *
     * @ORM\ManyToOne(targetEntity="TContrato")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="contrato_id", referencedColumnName="id")
     * })
     */
    private $contrato;

    /**
     * @var \TCategoria
     *
     * @ORM\ManyToOne(targetEntity="TCategoria")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="categoria_id", referencedColumnName="id")
     * })
     */
    private $categoria;

    /**
     * @var \TJornada
     *
     * @ORM\ManyToOne(targetEntity="TJornada")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="jornada_id", referencedColumnName="id")
     * })
     */
    private $jornada;

    /**
     * @var \TNivel
     *
     * @ORM\ManyToOne(targetEntity="TNivel")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="nivel_id", referencedColumnName="id")
     * })
     */
    private $nivel;

    /**
     * @var \TPersonal
     *
     * @ORM\ManyToOne(targetEntity="TPersonal")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="personal_id", referencedColumnName="id")
     * })
     */
    private $personal;

    /**
     * @var int
     *
     * @ORM\Column(name="horas", type="integer", nullable=true)
     */
    private $horas;


    /**
     * @var \DateTime|null
     *
     * @ORM\Column(name="fecha_alta", type="datetime", nullable=true)
     */
    private $fechaAlta;

    /**
     * @var \DateTime|null
     *
     * @ORM\Column(name="fecha_baja", type="datetime", nullable=true)
     */
    private $fechaBaja;

    /**
     * @var string
     *
     * @ORM\Column(name="puesto", type="string", nullable=true)
     */
    private $puesto;

    /**
     * @var \DateTime|null
     *
     * @ORM\Column(name="prueba", type="datetime", nullable=true)
     */
    private $prueba;

    /**
     * @var string
     *
     * @ORM\Column(name="rendimiento", type="string", nullable=true)
     */
    private $rendimiento;

    /**
     * @var int
     *
     * @ORM\Column(name="relacion", type="integer", nullable=true)
     */
    private $relacion;

    /**
     * @var \App\Entity\TModalidad
     *
     * @ORM\ManyToOne(targetEntity="App\Entity\TModalidad")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="modalidad", referencedColumnName="id")
     * })
     */
    private $modalidad;

    /**
     * @var \TPersonal
     *
     * @ORM\ManyToOne(targetEntity="TPersonal")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="sustituto", referencedColumnName="id")
     * })
     */
    private $sustituto;

    /**
     * @var int
     *
     * @ORM\Column(name="tipo", type="integer", nullable=true)
     */
    private $tipo;


    /**
     * @var \App\Entity\TDepartamento
     *
     * @ORM\ManyToOne(targetEntity="App\Entity\TDepartamento")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="dpto_pas", referencedColumnName="id")
     * })
     */
    private $dpto;


    /**
     * @var int
     *
     * @ORM\Column(name="causa_baja", type="integer", nullable=true)
     */
    private $causa;


    /**
     * @var int
     *
     * @ORM\Column(name="causa_alta", type="integer", nullable=true)
     */
    private $alta;

    /**
     * @var \App\Entity\TCurso
     *
     * @ORM\ManyToOne(targetEntity="App\Entity\TCurso")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="curso_id", referencedColumnName="id")
     * })
     */
    private $cursoAcada;

    private $aDatosEconomicos;



    /**
     * Get the value of id
     *
     * @return  int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set the value of id
     *
     * @param  int  $id
     *
     * @return  self
     */
    public function setId($id)
    {
        $this->id = $id;

        return $this;
    }

    /**
     * Get the value of contrato
     *
     * @return  \TContrato
     */
    public function getContrato()
    {
        return $this->contrato;
    }

    /**
     * Set the value of contrato
     *
     * @param  \TContrato  $contrato
     *
     * @return  self
     */
    public function setContrato($contrato)
    {
        $this->contrato = $contrato;

        return $this;
    }

    /**
     * Get the value of jornada
     *
     * @return  \TJornada
     */
    public function getJornada()
    {
        return $this->jornada;
    }

    /**
     * Set the value of jornada
     *
     * @param  \TJornada  $jornada
     *
     * @return  self
     */
    public function setJornada($jornada)
    {
        $this->jornada = $jornada;

        return $this;
    }

    /**
     * Get the value of nivel
     *
     * @return  \TNivel
     */
    public function getNivel()
    {
        return $this->nivel;
    }

    /**
     * Set the value of nivel
     *
     * @param  \TNivel  $nivel
     *
     * @return  self
     */
    public function setNivel($nivel)
    {
        $this->nivel = $nivel;

        return $this;
    }

    /**
     * Get the value of personal
     *
     * @return  \TPersonal
     */
    public function getPersonal()
    {
        return $this->personal;
    }

    /**
     * Set the value of personal
     *
     * @param  \TPersonal  $personal
     *
     * @return  self
     */
    public function setPersonal($personal)
    {
        $this->personal = $personal;

        return $this;
    }

    /**
     * Get the value of horas
     *
     * @return  int
     */
    public function getHoras()
    {
        return $this->horas;
    }

    /**
     * Set the value of horas
     *
     * @param  int  $horas
     *
     * @return  self
     */
    public function setHoras($horas)
    {
        $this->horas = $horas;

        return $this;
    }

    /**
     * Get the value of fechaAlta
     *
     * @return  \DateTime|null
     */
    public function getFechaAlta()
    {
        return $this->fechaAlta;
    }

    /**
     * Set the value of fechaAlta
     *
     * @param  \DateTime|null  $fechaAlta
     *
     * @return  self
     */
    public function setFechaAlta($fechaAlta)
    {
        $this->fechaAlta = $fechaAlta;

        return $this;
    }

    /**
     * Get the value of fechaBaja
     *
     * @return  \DateTime|null
     */
    public function getFechaBaja()
    {
        return $this->fechaBaja;
    }

    /**
     * Set the value of fechaBaja
     *
     * @param  \DateTime|null  $fechaBaja
     *
     * @return  self
     */
    public function setFechaBaja($fechaBaja)
    {
        $this->fechaBaja = $fechaBaja;

        return $this;
    }

    /**
     * Get the value of categoria
     *
     * @return  \TCategoria
     */
    public function getCategoria()
    {
        return $this->categoria;
    }

    /**
     * Set the value of categoria
     *
     * @param  \TCategoria  $categoria
     *
     * @return  self
     */
    public function setCategoria($categoria)
    {
        $this->categoria = $categoria;

        return $this;
    }

    /**
     * Get the value of puesto
     *
     * @return  string
     */
    public function getPuesto()
    {
        return $this->puesto;
    }

    /**
     * Set the value of puesto
     *
     * @param  string  $puesto
     *
     * @return  self
     */
    public function setPuesto($puesto)
    {
        $this->puesto = $puesto;

        return $this;
    }

    /**
     * Get the value of prueba
     *
     * @return  \DateTime|null
     */
    public function getPrueba()
    {
        return $this->prueba;
    }

    /**
     * Set the value of prueba
     *
     * @param  \DateTime|null  $prueba
     *
     * @return  self
     */
    public function setPrueba($prueba)
    {
        $this->prueba = $prueba;

        return $this;
    }

    /**
     * Get the value of rendimiento
     *
     * @return  string
     */
    public function getRendimiento()
    {
        return $this->rendimiento;
    }

    /**
     * Set the value of rendimiento
     *
     * @param  string  $rendimiento
     *
     * @return  self
     */
    public function setRendimiento($rendimiento)
    {
        $this->rendimiento = $rendimiento;

        return $this;
    }

    /**
     * Get the value of relacion
     *
     * @return  int
     */
    public function getRelacion()
    {
        return $this->relacion;
    }

    /**
     * Set the value of relacion
     *
     * @param  int  $relacion
     *
     * @return  self
     */
    public function setRelacion($relacion)
    {
        $this->relacion = $relacion;

        return $this;
    }

    /**
     * Get the value of modalidad
     *
     * @return  \TModalidad
     */
    public function getModalidad()
    {
        return $this->modalidad;
    }

    /**
     * Set the value of modalidad
     *
     * @param  \TModalidad  $modalidad
     *
     * @return  self
     */
    public function setModalidad($modalidad)
    {
        $this->modalidad = $modalidad;

        return $this;
    }

    /**
     * Get the value of sustituto
     *
     * @return  \TPersonal
     */
    public function getSustituto()
    {
        return $this->sustituto;
    }

    /**
     * Set the value of sustituto
     *
     * @param  \TPersonal  $sustituto
     *
     * @return  self
     */
    public function setSustituto($sustituto)
    {
        $this->sustituto = $sustituto;

        return $this;
    }

    /**
     * Get the value of tipo
     *
     * @return  int
     */
    public function getTipo()
    {
        return $this->tipo;
    }

    /**
     * Set the value of tipo
     *
     * @param  int  $tipo
     *
     * @return  self
     */
    public function setTipo($tipo)
    {
        $this->tipo = $tipo;

        return $this;
    }

    /**
     * Get the value of dpto
     *
     * @return  \TDepartamento
     */
    public function getDpto()
    {
        return $this->dpto;
    }

    /**
     * Set the value of dpto
     *
     * @param  \TDepartamento  $dpto
     *
     * @return  self
     */
    public function setDpto($dpto)
    {
        $this->dpto = $dpto;

        return $this;
    }

    /**
     * Get the value of aDatosEconomicos
     */
    public function getADatosEconomicos()
    {
        return $this->aDatosEconomicos;
    }

    /**
     * Set the value of aDatosEconomicos
     *
     * @return  self
     */
    public function setADatosEconomicos($aDatosEconomicos)
    {
        $this->aDatosEconomicos = $aDatosEconomicos;

        return $this;
    }

    /**
     * Get the value of causa
     *
     * @return  int
     */
    public function getCausa()
    {
        return $this->causa;
    }

    /**
     * Set the value of causa
     *
     * @param  int  $causa
     *
     * @return  self
     */
    public function setCausa($causa)
    {
        $this->causa = $causa;

        return $this;
    }

    /**
     * Get the value of alta
     *
     * @return  int
     */
    public function getAlta()
    {
        return $this->alta;
    }

    /**
     * Set the value of alta
     *
     * @param  int  $alta
     *
     * @return  self
     */
    public function setAlta($alta)
    {
        $this->alta = $alta;

        return $this;
    }

    /**
     * Get the value of cursoAcada
     *
     * @return  \TCurso
     */ 
    public function getCursoAcada()
    {
        return $this->cursoAcada;
    }

    /**
     * Set the value of cursoAcada
     *
     * @param  \TCurso  $cursoAcada
     *
     * @return  self
     */ 
    public function setCursoAcada($cursoAcada)
    {
        $this->cursoAcada = $cursoAcada;

        return $this;
    }
}
