<?php

namespace App\Entity;

use Doctrine\ORM\Mapping as ORM;

/**
 * TFlujo
 *
 * @ORM\Table(name="flujo_t", schema="unam_ga")
 * @ORM\Entity
 */
class TFlujo
{
    const K_ENTIDAD = 'App\Entity\TFlujo';

    const K_ESTADOS = ['C' => 'Pendiente del coordinador', 'D' => 'Pendiente del decanato', 'A' => 'Pendiente de Rectorado/Calidad', 'R1' => 'Revisado por decanato', 'R2' => 'Revisado por decanato', 'R' => 'Revisado por decanato'];

    const K_DECANATO = 'D';
    const K_COORDINA = 'C';
    const K_RECTORADO = 'R';
    const K_RESPUESTA1 = 'R1';
    const K_RESPUESTA2 = 'R2';
    const K_CALIDAD = 'A';

    const K_VALIDA_OD = 1;
    const K_RECHAZA_OD = 0;

    /**
     * @var int
     *
     * @ORM\Column(name="id", type="integer", nullable=false)
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="SEQUENCE")
     * @ORM\SequenceGenerator(sequenceName="unam_ga.facultad_t_id_seq", allocationSize=1, initialValue=1)
     */
    private $id;

    /**
     * @var \Tusuarios
     *
     * @ORM\ManyToOne(targetEntity="Tusuarios")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="personal_id", referencedColumnName="id")
     * })
     */
    private $usuario;


    /**
     * @var \TCurso
     *
     * @ORM\ManyToOne(targetEntity="TCurso")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="curso_id", referencedColumnName="id")
     * })
     */
    private $cursoAcada;

    /**
     * @var \TFacultad
     *
     * @ORM\ManyToOne(targetEntity="TFacultad")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="facultad_id", referencedColumnName="id")
     * })
     */
    private $facultad;

    /**
     * @var \DateTime|null
     *
     * @ORM\Column(name="fechaauditoria", type="datetime", nullable=true)
     */
    private $fechaAuditoria;

    /**
     * @var string|null
     *
     * @ORM\Column(name="accion", type="text", nullable=false)
     */
    private $texto;


    /**
     * @var string
     *
     * @ORM\Column(name="estado", type="string", nullable=false)
     */
    private $estado;

    /**
     * @ORM\Column(name="pdf",type="blob", nullable=true)
     */
    private $archivo;

    /**
     * @var int
     *
     * @ORM\Column(name="respuesta", type="integer", nullable=false)
     */
    private $respuesta;

    /**
     * Get the value of id
     *
     * @return  int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set the value of id
     *
     * @param  int  $id
     *
     * @return  self
     */
    public function setId($id)
    {
        $this->id = $id;

        return $this;
    }

    /**
     * Get the value of usuario
     *
     * @return  \Tusuarios
     */
    public function getUsuario()
    {
        return $this->usuario;
    }

    /**
     * Set the value of usuario
     *
     * @param  \Tusuarios  $usuario
     *
     * @return  self
     */
    public function setUsuario($usuario)
    {
        $this->usuario = $usuario;

        return $this;
    }

    /**
     * Get the value of cursoAcada
     *
     * @return  \TCurso
     */
    public function getCursoAcada()
    {
        return $this->cursoAcada;
    }

    /**
     * Set the value of cursoAcada
     *
     * @param  \TCurso  $cursoAcada
     *
     * @return  self
     */
    public function setCursoAcada($cursoAcada)
    {
        $this->cursoAcada = $cursoAcada;

        return $this;
    }

    /**
     * Get the value of facultad
     *
     * @return  \TFacultad
     */
    public function getFacultad()
    {
        return $this->facultad;
    }

    /**
     * Set the value of facultad
     *
     * @param  \TFacultad  $facultad
     *
     * @return  self
     */
    public function setFacultad($facultad)
    {
        $this->facultad = $facultad;

        return $this;
    }

    /**
     * Get the value of fechaAuditoria
     *
     * @return  \DateTime|null
     */
    public function getFechaAuditoria()
    {
        return $this->fechaAuditoria;
    }

    /**
     * Set the value of fechaAuditoria
     *
     * @param  \DateTime|null  $fechaAuditoria
     *
     * @return  self
     */
    public function setFechaAuditoria($fechaAuditoria)
    {
        $this->fechaAuditoria = $fechaAuditoria;

        return $this;
    }

    /**
     * Get the value of texto
     *
     * @return  string|null
     */
    public function getTexto()
    {
        return $this->texto;
    }

    /**
     * Set the value of texto
     *
     * @param  string|null  $texto
     *
     * @return  self
     */
    public function setTexto($texto)
    {
        $this->texto = $texto;

        return $this;
    }

    /**
     * Get the value of estado
     *
     * @return  string
     */
    public function getEstado()
    {
        return $this->estado;
    }

    /**
     * Set the value of estado
     *
     * @param  string  $estado
     *
     * @return  self
     */
    public function setEstado($estado)
    {
        $this->estado = $estado;

        return $this;
    }

    /**
     * Get the value of archivo
     */
    public function getArchivo()
    {
        return $this->archivo;
    }

    /**
     * Set the value of archivo
     *
     * @return  self
     */
    public function setArchivo($archivo)
    {
        $this->archivo = $archivo;

        return $this;
    }

    /**
     * Get the value of respuesta
     *
     * @return  int
     */ 
    public function getRespuesta()
    {
        return $this->respuesta;
    }

    /**
     * Set the value of respuesta
     *
     * @param  int  $respuesta
     *
     * @return  self
     */ 
    public function setRespuesta($respuesta)
    {
        $this->respuesta = $respuesta;

        return $this;
    }
}
