<?php

/**
 * Clase ORM para la conexión con la BD
 */

namespace App\ORM;

use App\Entity\GP\TJornada;
use Doctrine\Common\Persistence\ManagerRegistry;
use Symfony\Component\DependencyInjection\ContainerInterface;
use \Doctrine\Common\Persistence\ObjectManager;
use Symfony\Component\Security\Core\Authorization\AuthorizationCheckerInterface;
use \Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\EntityManagerInterface;
use \Exception;
use DateTimeImmutable;
use DateTime;

use App\Entity\Tusuarios;
use App\Entity\TFacultad;
use App\Entity\TLog;
use App\Entity\TModalidad;
use App\Entity\TPlan;
use App\Entity\TTitulacion;
use App\Entity\TDepartamento;
use App\Entity\TTitulacionMaestro;
use App\Entity\TAsignatura;
use App\Entity\TAsignaturaCurso;
use App\Entity\TAsignaturaMaestro;
use App\Entity\TCurso;
use App\Entity\GP\TPersonalalta;
use App\Entity\GP\TPersonal;
use App\Entity\TAsignaturaDual;
use App\Entity\TAsignaturaPdi;
use App\Entity\TDptoPdi;
use App\Entity\TFlujo;

/**
 * Class Orm
 *
 */
class ORM
{

    private $entityManager;

    public function __construct(EntityManagerInterface $entityManager)
    {
        $this->entityManager = $entityManager;
    }

    /************************************************************
     * LOGIN
     */
    /**
     * comprueba que un usuario exista en la BD
     */
    public function comprobarUsuarioBD($nombre, $clave, &$sError)
    {
        try {
            $oUsuario = $this->entityManager->getRepository(Tusuarios::class)->findOneBy(array('nombre' => $nombre, 'password' => $clave));

            if (is_object($oUsuario)) {

                switch ($oUsuario->getRol()) {
                    case 1:
                        $rol = ['GA.ADMIN'];
                        break;
                    case 2:
                        $rol = ['GA.TECNICO'];
                        break;
                    case 3:
                        $rol = ['GA.USUARIO'];
                        break;
                    case 4:
                        $rol = ['GA.SUPER'];
                        break;
                    default:
                        $rol = ['GA.USUARIO'];
                }

                $oUsuario->setRoles($rol);

                return $oUsuario;
            } else return false;
        } catch (Exception $e) {
            $sError .= $e->getMessage();
            return 0;
        }
    }

    /**
     * comprueba que un usuario exista en la BD
     */
    public function comprobarUsuarioMSLoginBD($mail,  &$sError)
    {
        try {
            $oUsuario = $this->entityManager->getRepository(Tusuarios::class)->findOneBy(array('email' => $mail));

            if (is_object($oUsuario)) {
                //si se encuentra, guardamos su rol
                $oUsuario->setRoles([Tusuarios::ROLTXT[$oUsuario->getRol()]]);
                return $oUsuario;
            } else {
                //no tiene permisos para consultar la app
                $sError = true;
            }
        } catch (Exception $e) {
            $sError .= $e->getMessage();
            throw new Exception(" $sError ");
            return 0;
        }
    }


    /************************************************************
     * USUARIOS
     */

    /***
     * devuelve uno o todos los usuarios
     */
    public function obtenerUsuarios($id = null, $idCategoria = null, $rol = null)
    {
        if ($rol && $idCategoria)
            $oDato = $this->entityManager->getRepository(Tusuarios::class)->findBy(['categoria' => $idCategoria, 'rol' => $rol]);
        elseif ($rol) {
            $oDato = $this->entityManager->getRepository(Tusuarios::class)->findBy(['rol' => $rol]);
        } elseif ($idCategoria) {
            $oDato = $this->entityManager->getRepository(Tusuarios::class)->findBy(['categoria' => $idCategoria]);
        } elseif ($id) {
            $oDato = $this->entityManager->getRepository(Tusuarios::class)->find($id);
        } else {
            $oDato = $this->entityManager->getRepository(Tusuarios::class)->findAll();
        }

        return $oDato;
    }

    /**
     * guarda o modifica un usuario
     */
    public function modificarUsuarios($request, &$sError)
    {
        $idUsuario = $request->get('id', NULL); //para el caso de modificar
        $sError       = FALSE;
        $sNombre      = $request->get('nombre', NULL);
        $sMail        = $request->get('mail', NULL);
        $sTlf        = $request->get('tlf', NULL);
        $nRol     = $request->get('rol', NULL);
        $nFacultad = $request->get('dpto', NULL);
        $nTipo = $request->get('tipo', NULL);

        try {
            $oFacultad = $this->obtenerEntidad($nFacultad, TFacultad::K_ENTIDAD);

            //modificando -> obtenemos la entidad a modificar
            if ($idUsuario) {
                $oUsuario = $this->obtenerEntidad($idUsuario, Tusuarios::K_ENTIDAD);
            } else {
                $oUsuario = new Tusuarios();
            }
            $oUsuario->setNombre($sNombre);
            $oUsuario->setEmail($sMail);
            $oUsuario->setTlf($sTlf);
            $oUsuario->setFacultad($oFacultad);
            $oUsuario->setRol($nRol);
            $oUsuario->setTipo($nTipo);
            $this->grabarObjetoBD($oUsuario, $sError);

            return true;
        } catch (Exception $e) {
            $sError = $e->getMessage();
            return false;
        }
    }

    /**
     *  Se guarda un usuario que ha llegado de MS y no se ha dado de alta en la app
     */
    public function guardaUsuarioMS($email, &$sError = '')
    {
        list($nombreUsuario, $dominio) = explode('@', $email);
        $oDpto = 3; //TODO       
        try {

            $oUsuario = new Tusuarios();
            $oUsuario->setNombre($nombreUsuario);
            $oUsuario->setEmail($email);
            $oUsuario->setRol(9); //TODO

            //$this->grabarObjetoBD($oUsuario, $sError);

            return $oUsuario;
        } catch (Exception $e) {
            $sError = $e->getMessage();
            throw new Exception(" $sError ");
            return false;
        }
        return false;
    }


    /************************************************************
     * Facultad
     */


    /**
     * guarda o modifica 
     */
    public function modificarfacultad($request, $sEntidad, &$sError)
    {
        $id = $request->get('id', NULL); //para el caso de modificar
        $sError      = FALSE;
        $sNombre     = $request->get('nombre', NULL);
        $sMail     = $request->get('mail', NULL);

        try {
            //modificando -> obtenemos la entidad a modificar
            if ($id) {
                $oDato = $this->obtenerEntidad($id, $sEntidad);
            } else {
                $oDato = new TFacultad();
                $oDato->setActivo(TFacultad::K_ACTIVO);
            }

            $oDato->setNombre($sNombre);
            $oDato->setMail($sMail);

            $this->grabarObjetoBD($oDato, $sError);

            return true;
        } catch (Exception $e) {
            throw new \Exception($e);
            $sError = $e->getMessage();
            return false;
        }
    }



    /************************************************************
     * Modalidad
     */


    /**
     * guarda o modifica 
     */
    public function modificarmodalidad($request, $sEntidad, &$sError)
    {
        $id = $request->get('id', NULL); //para el caso de modificar
        $sError      = FALSE;
        $sNombre     = $request->get('nombre', NULL);
        $nHoras =  $request->get('horas', NULL);

        try {
            //modificando -> obtenemos la entidad a modificar
            if ($id) {
                $oDato = $this->obtenerEntidad($id, $sEntidad);
            } else {
                $oDato = new TModalidad();
            }

            $oDato->setNombre($sNombre);
            $oDato->setHoras($nHoras);

            $this->grabarObjetoBD($oDato, $sError);

            return true;
        } catch (Exception $e) {
            throw new \Exception($e);
            $sError = $e->getMessage();
            return false;
        }
    }



    /************************************************************
     * Titulaci.
     */
    /**
     * guarda o modifica 
     */
    public function modificartitulacion($request, $sEntidad, &$sError)
    {
        $id = $request->get('id', NULL); //para el caso de modificar
        $sError      = FALSE;
        $nTitulo     = $request->get('titulo', NULL);
        $nFacultad     = $request->get('facultad', NULL);
        $nModalidad     = $request->get('modalidad', NULL);
        $nPlan     = $request->get('plan', NULL);
        $nActivo     = $request->get('activo', 1);

        try {
            //modificando -> obtenemos la entidad a modificar
            if ($id) {
                $oDato = $this->obtenerEntidad($id, $sEntidad);
            } else {
                $oDato = new TTitulacion();
            }

            $oDato->setTitulo($this->obtenerEntidad($nTitulo, TTitulacionMaestro::K_ENTIDAD));
            $oDato->setActivo($nActivo);
            $oDato->setFacultad($this->obtenerEntidad($nFacultad, TFacultad::K_ENTIDAD));
            $oDato->setModalidad($this->obtenerEntidad($nModalidad, TModalidad::K_ENTIDAD));
            $oDato->setPlan($this->obtenerEntidad($nPlan, TPlan::K_ENTIDAD));

            $this->grabarObjetoBD($oDato, $sError);

            return true;
        } catch (Exception $e) {
            throw new \Exception($e);
            $sError = $e->getMessage();
            return false;
        }
    }

    /************************************************************
     * Asignatura
     */
    /**
     * guarda o modifica 
     */
    public function modificarasignatura($request, $sEntidad, &$sError)
    {
        $id = $request->get('id', NULL); //para el caso de modificar
        $sError      = FALSE;
        $nAsigna     = $request->get('asignatura', NULL);
        $nCreditos     = $request->get('creditos', NULL);
        $nCreditosAlu     = $request->get('creditosAlu', NULL);
        $nHoras     = $request->get('horas', NULL);
        $nTitulo     = $request->get('titulacion', NULL);
        $nActivo     = $request->get('activo', 1);
        $sTipo    = $request->get('tipo', 1);
        $nPlan    = $request->get('plan', null);
        $nModalidad    = $request->get('modalidad', null);

        try {
            //modificando -> obtenemos la entidad a modificar
            if ($id) {
                $oDato = $this->obtenerEntidad($id, $sEntidad);
            } else {
                $oDato = new TAsignatura();
                $oDato->setDidactica('T');
            }

            $oTitulo = $this->entityManager->getRepository(TTitulacion::class)->findOneBy(['modalidad' => $nModalidad, 'plan' => $nPlan, 'titulo' => $nTitulo]);

            $oDato->setAsignatura($this->obtenerEntidad($nAsigna, TAsignaturaMaestro::K_ENTIDAD));
            $oDato->setTitulacion($this->obtenerEntidad($nTitulo, TTitulacionMaestro::K_ENTIDAD));
            $oDato->setModalidad($this->obtenerEntidad($nModalidad, TModalidad::K_ENTIDAD));

            /*OJO - SI SE IMPORTA DESDE FICHERO HAY Q RELLENAR A MANO EL CAMPO -- PROBAR POR TRIGGER Q ES LO IDEAL
            UPDATE unam_ga.asignatura_t SET titulo_plan_modalidad_id = amt.id
            FROM unam_ga.titulacion_t amt WHERE asignatura_t.titulacion_id  = amt.titulo_id 
            and asignatura_t.modalidad_id  = amt.modalidad_id and asignatura_t.plan_id  = amt.plan_id ;
            */
            $oDato->setTitulo_plan_modalidad($oTitulo);

            $oDato->setPlan($this->obtenerEntidad($nPlan, TPlan::K_ENTIDAD));
            //$oDato->setActivo($nActivo);
            $oDato->setCreditos($nCreditos);
            $oDato->setCreditosAlumno($nCreditosAlu);

            $oDato->setHoras($nHoras);

            $oDato->setTipo($sTipo);

            $this->grabarObjetoBD($oDato, $sError);

            return true;
        } catch (Exception $e) {
            $sError .= $e->getMessage();
            throw new \Exception($e);
            return false;
        }
    }

    /************************************************************
     * Asignatura DUAL
     */
    /**
     * guarda o modifica 
     */
    public function crearAsignaturaDual($request, $nTitulo, $nCurso, &$sError)
    {
        $sError      = '';
        $nAsigna     = $request->get('asignatura', NULL);

        try {

            $oDato = new TAsignaturaDual();

            $codAdignaturaDual = $this->obtenerasignaturadual($nAsigna, $nTitulo, $nCurso, $sError);

            if ($codAdignaturaDual) {
                $oAsignatura = $this->entityManager->getRepository(TAsignaturaCurso::class)->find($nAsigna);
                $oAsignaturaDual = $this->entityManager->getRepository(TAsignaturaCurso::class)->find($codAdignaturaDual);

                $oDato->setAsignatura($oAsignatura);
                $oDato->setAsignaturaDual($oAsignaturaDual);

                $this->grabarObjetoBD($oDato, $sError);
                return true;
            } else {
                $sError = 'No se ha podido obtener la asignatura dual ' . $codAdignaturaDual . ' ' . $sError;
                return false;
            }
        } catch (Exception $e) {
            throw new \Exception($e);
            $sError = $e->getMessage();
            return false;
        }
    }



    /************************************************************
     * Asignatura - curso
     */
    /**
     * guarda o modifica 
     */
    public function modificarasignacurso($request, $sEntidad, &$sError)
    {
        $id = $request->get('id', NULL); //para el caso de modificar
        $sError      = FALSE;

        $ncursoAcada     = $request->get('cursoacada', NULL);
        $ncurso     = $request->get('curso', NULL);
        $nSemestre   = $request->get('semestre', NULL);
        $ngrupo    = $request->get('grupo', NULL);

        $nasignatura     = $request->get('asignatura', NULL);
        $nTitulo    = $request->get('titulo', NULL);
        $nModalidad    = $request->get('modalidad', NULL);
        $nPlan    = $request->get('plan', NULL);


        try {
            //modificando -> obtenemos la entidad a modificar
            if ($id) {
                $oDato = $this->obtenerEntidad($id, $sEntidad);
            } else {
                $oDato = new TAsignaturaCurso();
            }

            //si es un insert o modifica el admin.
            if ($nTitulo && $nModalidad && $nPlan) {
                $oAsignatura = $this->entityManager->getRepository(TAsignatura::class)->findOneBy(['titulacion' => $nTitulo, 'modalidad' => $nModalidad, 'plan' => $nPlan, 'asignatura' => $nasignatura]);
                $oDato->setAsignatura($oAsignatura);
                $oDato->setCursoAcada($this->obtenerEntidad($ncursoAcada, TCurso::K_ENTIDAD));
                $oDato->setCurso($ncurso);
                $oDato->setSemestre($nSemestre);
            }

            $oDato->setGrupos($ngrupo);

            $this->grabarObjetoBD($oDato, $sError);

            return true;
        } catch (Exception $e) {
            throw new \Exception($e);
            $sError = $e->getMessage();
            return false;
        }
    }


    /**
     * guarda o modifica 
     */
    public function modificarDptoPdi($request, $sEntidad, &$sError)
    {
        $id = $request->get('id', NULL); //para el caso de modificar
        $dpto = $request->get('dpto', NULL); //para el caso de modificar
        $pdi = $request->get('pdi', NULL); //para el caso de modificar
        $sError      = FALSE;


        try {
            //modificando -> obtenemos la entidad a modificar
            if ($id) {
                $oDato = $this->obtenerEntidad($id, $sEntidad);
            } else {
                $oDato = new $sEntidad();
            }

            $oDato->setDpto($this->obtenerEntidad($dpto, TDepartamento::K_ENTIDAD));
            $oDato->setPdi($this->obtenerEntidad($pdi, TPersonalalta::K_ENTIDAD));

            $this->grabarObjetoBD($oDato, $sError);

            return true;
        } catch (Exception $e) {
            throw new \Exception($e);
            $sError = $e->getMessage();
            return false;
        }
    }


    /**
     * guarda o modifica 
     */
    public function modificarPdiAsignatura($id, $nAsignaturaCurso, $nPdi, $nCreditos, $nGrupo, &$sError, $bAsignaPdi = false)
    {

        $sError      = FALSE;
        $oAsignaturaDual = NULL;

        $sEntidad = TAsignaturaPdi::K_ENTIDAD;

        //TODO -- Avisar en caso de modificar una vez validada la convocatoria

        try {

            $oDato = new $sEntidad();
            $oAsignaturaCurso = $this->obtenerEntidad($nAsignaturaCurso, TAsignaturaCurso::K_ENTIDAD);
            $nEsDual = $this->obtenerSiEsAsignaturaDual($nAsignaturaCurso, $sError);
            $nCreditosTotales = $oAsignaturaCurso->getAsignatura()->getCreditos();

            if ($bAsignaPdi) { //si es asignar por pdi, comprobamos que exista
                $oTasignaturaPdi = $this->entityManager->getRepository(TAsignaturaPdi::K_ENTIDAD)->findOneBy(['grupo' => $nGrupo, 'asignatura' => $nAsignaturaCurso, 'pdi' => $nPdi, 'cursoAcada' => $oAsignaturaCurso->getCursoAcada()->getId()]);
                if ($oTasignaturaPdi instanceof TAsignaturaPdi) {
                    //si existe, actualizamos
                    $oDato = $oTasignaturaPdi;
                }
            } //modificando -> obtenemos la entidad a modificar

            if ($id) {
                $oDato = $this->obtenerEntidad($id, $sEntidad);
            }

            $oPersonalAlta = $this->entityManager->getRepository(TPersonalalta::K_ENTIDAD)->findOneBy(['id' => $nPdi]);

            $oDato->setAsignatura($oAsignaturaCurso);
            $oDato->setPdi($oPersonalAlta);
            //al guardar varias de una vez, no llegan ya que es el total de la asignatura
            $oDato->setCreditos($nCreditos ? $nCreditos : $nCreditosTotales); // TODO -- posible error en caso de no llegar un int
            $oDato->setGrupo($nGrupo);

            $this->grabarObjetoBD($oDato, $sError);

            if ($nEsDual) {
                //en caso de ser DUAL, actualizamos la DUAL --TODO --  OJO AL ELIMINAR
                $oAsignaturaDual = $this->crearAsignaturaDualPDI($nEsDual, $nPdi, $nCreditos);
                $this->grabarObjetoBD($oAsignaturaDual, $sError);
            }


            return true;
        } catch (Exception $e) {
            throw new \Exception($e);
            $sError = $e->getMessage();
            return false;
        }
    }

    //para el caso de eliminar, debemos localizar la DUAL
    public function localizarAsignaturaPDI($nAsignatura, $nPdi, $cursoAcada, $nGrupo)
    {

        $oTasignaturaPdi = $this->entityManager->getRepository(TAsignaturaPdi::K_ENTIDAD)->findOneBy(['grupo' => $nGrupo, 'asignatura' => $nAsignatura, 'pdi' => $nPdi, 'cursoAcada' => $cursoAcada]);

        return $oTasignaturaPdi;
    }

    private function crearAsignaturaDualPDI($nAsignatura, $nPdi, $nCreditos)
    {

        //obtenemos la asignatura DUAL
        $oAsignaturaCurso = $this->obtenerEntidad($nAsignatura, TAsignaturaCurso::K_ENTIDAD);
        //obtenemos los grupos de la asignatura
        $nGrupo = $oAsignaturaCurso->getGrupos();
        //si es dual es +1
        $nGrupo++;

        //obtenemos los datos del docente
        $oPersonalAlta = $this->entityManager->getRepository(TPersonalalta::K_ENTIDAD)->findOneBy(['id' => $nPdi]);

        //creamos el objeto
        $sEntidad = TAsignaturaPdi::K_ENTIDAD;
        $oDato = new $sEntidad();

        //vamos a comprobar si ya existen datos para ese docente de esa asignatura
        $oTasignaturaPdi = $this->entityManager->getRepository(TAsignaturaPdi::K_ENTIDAD)->findOneBy(['grupo' => $nGrupo, 'asignatura' => $nAsignatura, 'pdi' => $nPdi, 'cursoAcada' => $oAsignaturaCurso->getCursoAcada()->getId()]);

        if ($oTasignaturaPdi instanceof TAsignaturaPdi) {
            //si existe, actualizamos
            $oDato = $oTasignaturaPdi;
        }
        $oDato->setAsignatura($oAsignaturaCurso);
        $oDato->setPdi($oPersonalAlta);
        $oDato->setCreditos($nCreditos);
        $oDato->setGrupo($nGrupo);

        return $oDato;
    }

    /************************************************************     
     */


    /**
     * guarda o modifica 
     */
    public function modificar($request, $sEntidad, &$sError)
    {
        $id = $request->get('id', NULL); //para el caso de modificar
        $sError      = FALSE;
        $sNombre     = $request->get('nombre', NULL);
        $sActivo     = $request->get('activo', NULL);

        try {
            //modificando -> obtenemos la entidad a modificar
            if ($id) {
                $oDato = $this->obtenerEntidad($id, $sEntidad);
            } else {
                $oDato = new $sEntidad();
            }

            if ($sNombre) {
                $oDato->setNombre($sNombre);
            }

            if ($sActivo) {
                $oDato->setActivo($sActivo == 'S' ? 1 : 0);
            }

            $this->grabarObjetoBD($oDato, $sError);

            return true;
        } catch (Exception $e) {
            throw new \Exception($e);
            $sError = $e->getMessage();
            return false;
        }
    }

    /************************************************************       /
     *************************************************************       /
     *************************************************************       / 

    /**
     * Eliminar
     */
    public function eliminar($id, $sEntidad, &$sError)
    {

        $oDato = $this->entityManager->getRepository($sEntidad)->find($id);
        try {
            $this->borrarObjetoBD($oDato, $sError);
        } catch (Exception $e) {
            $sError = $e->getMessage();
        }
        return true;
    }

    //obtener datos de una entidad
    public function obtenerEntidad($id = null, $sEntidad = null, $bOrden = false)
    {
        //ORM::depurar(func_get_args());
        if ($id) {
            $oDato = $this->entityManager->getRepository($sEntidad)->find($id);
            return $oDato;
        } elseif ($bOrden) {
            $aDatos = $this->entityManager->getRepository($sEntidad)->findBy([], ['nombre' => 'ASC']);
            return $aDatos;
        } else {
            $aDatos = $this->entityManager->getRepository($sEntidad)->findAll();
            return $aDatos;
        }
    }

    //obtener los titu. de un curso acad
    public function obtenerPDICurso($nCursoAcada, $nFacultad = null)
    {

        $aDatos = $this->entityManager->getRepository(TAsignaturaPdi::class)->findBy(['cursoAcada' => $nCursoAcada]);
        $aResultados = [];

        //si llega facultad, solamente las asignaturas de esa facultad
        if ($nFacultad) {
            foreach ($aDatos as  $value) {
                $Facultad = $value->getAsignatura()->getAsignatura()->getTitulo_plan_modalidad()->getFacultad()->getId();
                if ($nFacultad == $Facultad) {
                    $aResultados[] = $value;
                }
            }
        } else {
            $aResultados = $aDatos;
        }

        return $aResultados;
    }


    //obtener los titu. de un curso acad
    public function obtenerDualCurso($nCursoAcada)
    {
        $aDatos = $this->entityManager->getRepository(TAsignaturaDual::class)->findAll();
        $aDualesCurso  = [];

        foreach ($aDatos as  $value) {
            //solamente las duales del curso // TODO -- REVISAR PARA HACER DIRECTAMENTE SOBRE BD

            if ($value->getAsignatura()->getCursoAcada()->getId() == $nCursoAcada) {
                $aDualesCurso[] = $value;
            }
        }
        return $aDualesCurso;
    }

    //obtener los titu. de un curso acad
    public function obtenerTitulosCurso($nCursoAcada, $filtrado = null)
    {
        //$aDatos = $this->entityManager->getRepository(TAsignaturaCurso::class)->findBy(['cursoAcada' => $nCursoAcada]);return $aDatos;

        $idTitulo = isset($filtrado['tituloModificado']) ? $filtrado['tituloModificado'] : null;
        $nModalidad = isset($filtrado['modalidadModificado']) ? $filtrado['modalidadModificado'] : null;
        $nPlan = isset($filtrado['planModificado']) ? $filtrado['planModificado'] : null;
        $nFacultad = isset($filtrado['facultadModificado']) ? $filtrado['facultadModificado'] : null;

        $queryBuilder = $this->entityManager->createQueryBuilder();

        $queryBuilder
            ->select('act, at2')
            ->from('App\Entity\TAsignaturaCurso', 'act')
            ->join('act.asignatura', 'at2')
            ->join('at2.titulo_plan_modalidad', 'tpm')
            ->join('tpm.facultad', 'facultad')
            ->where('act.cursoAcada = :cursoId')
            ->andWhere('at2.titulacion != 43')
            ->setParameter('cursoId', $nCursoAcada);

        if ($nFacultad) {
            $queryBuilder->andWhere('facultad.id = :facultadId');
            $queryBuilder->setParameter('facultadId', $nFacultad);
        }

        if ($nModalidad) {
            $queryBuilder->andWhere('at2.modalidad = :modalidadId');
            $queryBuilder->setParameter('modalidadId', $nModalidad);
        }

        if ($nPlan) {
            $queryBuilder->andWhere('at2.plan = :planId');
            $queryBuilder->setParameter('planId', $nPlan);
        }

        if ($idTitulo) {
            $queryBuilder->andWhere('at2.titulacion = :titulacionId');
            $queryBuilder->setParameter('titulacionId', $idTitulo);
        }

        $query = $queryBuilder->getQuery();
        //self::depurar($queryBuilder->getDQL());
        return $query->getResult();
    }

    //obtener los titu. de un coordinados
    public function obtenerTitulosCoordinador($sEntidad, $nFacultad, $nCursoAcada, $filtrado = null)
    {
        $idTitulo = isset($filtrado['tituloModificado']) ? $filtrado['tituloModificado'] : null;
        $nModalidad = isset($filtrado['modalidadModificado']) ? $filtrado['modalidadModificado'] : null;
        $nPlan = isset($filtrado['planModificado']) ? $filtrado['planModificado'] : null;

        $queryBuilder = $this->entityManager->createQueryBuilder();

        $queryBuilder
            ->select('act, at2')
            ->from('App\Entity\TAsignaturaCurso', 'act')
            ->join('act.asignatura', 'at2')
            ->join('at2.titulo_plan_modalidad', 'tpm')
            ->join('tpm.facultad', 'facultad')
            ->where('facultad.id = :facultadId')
            ->andwhere('act.cursoAcada = :cursoId')
            ->andWhere('at2.titulacion != 43')
            ->setParameter('cursoId', $nCursoAcada)
            ->setParameter('facultadId', $nFacultad);

        if ($nModalidad) {
            $queryBuilder->andWhere('at2.modalidad = :modalidadId');
            $queryBuilder->setParameter('modalidadId', $nModalidad);
        }

        if ($nPlan) {
            $queryBuilder->andWhere('at2.plan = :planId');
            $queryBuilder->setParameter('planId', $nPlan);
        }

        if ($idTitulo) {
            $queryBuilder->andWhere('at2.titulacion = :titulacionId');
            $queryBuilder->setParameter('titulacionId', $idTitulo);
        }

        $query = $queryBuilder->getQuery();
        return $query->getResult();
    }

    //obtener los titu. de un coordinados
    public function obtenerTitulosCoordinadorPDI($sEntidad, $nFacultad, &$aSelect = null)
    {
        $aResultado = [];
        $aSelect = [];

        if ($nFacultad) {
            //TODO -- MEJORAR CON VM
            $aDatos = $this->entityManager->getRepository(TTitulacion::K_ENTIDAD)->findBy(['facultad' => $nFacultad]);
        } else {
            $aDatos = $this->entityManager->getRepository(TTitulacion::K_ENTIDAD)->findAll();
        }

        foreach ($aDatos as $titulacion) {
            if ($titulacion->getTitulo()->getId() != TAsignaturaCurso::TITULO_DUAL) {
                $aResultado[$titulacion->getTitulo()->getNombre()] = $titulacion->getTitulo();
                $aSelect[$titulacion->getTitulo()->getId()] = $titulacion->getTitulo()->getNombre();
            }
        }

        return $aResultado;
    }


    //obtener los titu. de un coordinados
    public function obtenerPlanTitulacion($nTitulo, $nModalidad = null)
    {
        $aResultado = [];
        if ($nModalidad) {
            $aDatos = $this->entityManager->getRepository(TTitulacion::K_ENTIDAD)->findBy(['titulo' => $nTitulo, 'modalidad' => $nModalidad]);
            foreach ($aDatos as $titulacion) {
                $aResultado[$titulacion->getPlan()->getId()] = $titulacion->getPlan()->getNombre();
            }
        } else {
            $aDatos = $this->entityManager->getRepository(TTitulacion::K_ENTIDAD)->findBy(['titulo' => $nTitulo]);
            foreach ($aDatos as $titulacion) {
                $aResultado[$titulacion->getPlan()->getId()] = $titulacion->getPlan()->getNombre();
            }
        }
        return $aResultado;
    }

    //obtiene todos los docentes asignados a un dpto.
    public function obtenerPDIbyDpto($nDpto = null, $nIdAcada = null)
    {
        if ($nDpto) {
            $aDatos = $this->entityManager->getRepository(TDptoPdi::K_ENTIDAD)->findBy(['dpto' => $nDpto]);
        } else {
            $aDatos = $this->entityManager->getRepository(TDptoPdi::K_ENTIDAD)->findAll();
        }
        $aActivos = [];
        $hoy = new DateTime();
        foreach ($aDatos as  $value) {
            //solamente los PDI(tipo 1) que que no tengan fecha de baja o la fecha de baja sea mayor que hoy
            //los que tengan curso o los que no tengan 
            if ( ($value->getPdi()->getTipo() == 1 ) &&
                 ( !$value->getPdi()->getFechaBaja() || $value->getPdi()->getFechaBaja() > $hoy ) &&
                 ( !$value->getPdi()->getCursoAcada() || $value->getPdi()->getCursoAcada()->getId() == $nIdAcada  )
                 ){
                $aActivos[] = $value;    
            } else{
               /* self::depurar("Curso -> $nIdAcada");
                self::depurar($value->getPdi()->getPersonal()->getNombre().' '.$value->getPdi()->getPersonal()->getApellido1(), true);                
                self::depurar($value->getPdi()->getTipo(), true);
                if ($value->getPdi()->getFechaBaja()) self::depurar($value->getPdi()->getFechaBaja()->format('d/m/Y') , true);
                if ($value->getPdi()->getCursoAcada()) self::depurar($value->getPdi()->getCursoAcada()->getId(), true);
                */
            }           
        }

        return $aActivos;
    }

    //obtiene todos las asignaturas asignadas a un PDI en un curso acad.
    public function obtenerAsignaturasPDI($nPdi, $nCursoAcada)
    {
        $aDatos = $this->entityManager->getRepository(TAsignaturaPdi::K_ENTIDAD)->findBy(['pdi' => $nPdi, 'cursoAcada' => $nCursoAcada]);
        $asignaturas = [];
        //self::depurar($aDatos, true);
        foreach ($aDatos as $asigna) {
            $asignaturas[$asigna->getAsignatura()->getId() . '_' . $asigna->getGrupo()] = $asigna;
        }
        return $asignaturas;
    }

    //obtener los titu. de un coordinados
    public function obtenerDocentesDpto($nDpto, $nIdAcada)
    {
        $aResultado = [];
        if ($nDpto) {
            //TODO -- MEJORAR CON VM
            //$aDatos = $this->entityManager->getRepository(TDptoPdi::K_ENTIDAD)->findBy(['dpto' => $nDpto]);
            $aDatos = $this->obtenerPDIbyDpto($nDpto);
            foreach ($aDatos as $dptoPdi) {
                $cadena = $dptoPdi->getPdi()->getPersonal()->getDni() . ' - ' . $dptoPdi->getPdi()->getPersonal()->getNombre() . ' ' . $dptoPdi->getPdi()->getPersonal()->getApellido1() . ' ' . $dptoPdi->getPdi()->getPersonal()->getApellido2();
                $cadena .= '@@@' . $dptoPdi->getPdi()->getHoras() . '@@@' . $dptoPdi->getPdi()->getJornada()->getNombre();

                $nIdPersonalAlta = $dptoPdi->getPdi()->getId();

                $suma = $this->sumarCreditosDocente($nIdPersonalAlta, $nIdAcada);

                $aResultado[$dptoPdi->getPdi()->getPersonal()->getId()] = $cadena . '@@@' . $suma;
            }
        }
        return $aResultado;
    }

    //obtener los titu. de un coordinados
    public function obtenerDocentesAsignatura($nAsigna)
    {
        $aDatos = $this->entityManager->getRepository(TAsignaturaPdi::K_ENTIDAD)->findBy(['asignatura' => $nAsigna]);
        return $aDatos;
    }



    /********************************************************************************************/
    //obtenemos todas las asignaturas de una titulac.
    public function obtenerAsignaturasTitulacion($id, $nModalidad = null, $nPlan = null)
    {
        $aResultado = [];
        //obtenemos las asignaturas de la titu.
        if ($nModalidad) {
            $aDato = $this->entityManager->getRepository(TAsignatura::class)->findBy(['titulacion' => $id, 'modalidad' => $nModalidad, 'plan' => $nPlan]);
        } else {
            $aDato = $this->entityManager->getRepository(TAsignatura::class)->findBy(['titulacion' => $id]);
        }

        //devolvemos el array con los datos
        foreach ($aDato as $value) {
            $dato['id'] = $value->getAsignatura()->getId();
            $dato['nombre'] = $value->getAsignatura()->getNombre();
            $aResultado[] = $dato;
        }

        usort($aResultado, function ($a, $b) {
            return strcmp($a['nombre'], $b['nombre']);
        });

        return $aResultado;
    }
    /************************************************************
     * LOG
     */

    //guardamos en el log 
    public function insertarLog($idUsuario, $nAccion, $sDatos, &$sError)
    {
        $oUsuario = $this->obtenerEntidad($idUsuario, Tusuarios::K_ENTIDAD);
        $oLog = new TLog();
        $oLog->setFecha(new \DateTime());
        $oLog->setAccion($nAccion);
        $oLog->setUsuario($oUsuario);
        $oLog->setDatos($sDatos);
        try {
            $this->grabarObjetoBD($oLog, $sError);

            return true;
        } catch (Exception $e) {
            throw new \Exception($e);
            $sError = $e->getMessage();
            return false;
        }
    }

    /************************************************************
     * Flujo de la app
     */
    public function obtenerFlujo($nCursoAcada, $nFacultad, $bValidar = false)
    {
        if ($bValidar) {
            //para comprobar que se ha validado la orden. docente, miramos si el estado es COORDINADOR con respuesta 1
            $aDatos = $this->entityManager->getRepository(TFlujo::class)->findBy(['cursoAcada' => $nCursoAcada, 'facultad' => $nFacultad, 'estado' => TFlujo::K_COORDINA, 'respuesta' => 1]);
        } elseif ($nFacultad) {
            $aDatos = $this->entityManager->getRepository(TFlujo::class)->findBy(['cursoAcada' => $nCursoAcada, 'facultad' => $nFacultad], ['fechaAuditoria' => 'DESC']);
        } else {
            $aDatos = $this->entityManager->getRepository(TFlujo::class)->findBy(['cursoAcada' => $nCursoAcada], ['fechaAuditoria' => 'DESC']);
        }

        return $aDatos;
    }

    /**
     * guarda o modifica 
     */
    public function modificarflujo($oUsuario, $aParam, &$sError)
    {
        $sError      = FALSE;


        $bEnviaCoordinador = $aParam['coordinador'] ?? false;
        $bEsCalidad = $aParam['calidad'] ??  false;
        $bEnviaRector = $aParam['rector'] ?? false;
        $bRechazar = $aParam['rechazo'] ?? false;

        $nIdAcada = $aParam['acada'];
        $sTexto = $aParam['texto'] ?? false;
        $fichero = $aParam['fichero'] ?? false;
        $nTipo = $aParam['tipo'] ?? false;
        $esFinal = $aParam['final'] ?? false;
        $nFacultad = isset($aParam['facultad']) ? $aParam['facultad'] : $oUsuario->getFacultad()->getId();


        try {
            $oFacultad = $this->obtenerEntidad($nFacultad, TFacultad::K_ENTIDAD);
            $oDato = new TFlujo();
            $oDato->setUsuario($this->obtenerEntidad($oUsuario->getId(), Tusuarios::K_ENTIDAD));
            $oDato->setCursoAcada($this->obtenerEntidad($nIdAcada, TCurso::K_ENTIDAD));
            $oDato->setFacultad($oFacultad);
            $oDato->setFechaAuditoria(new \DateTime());

            if ($bEnviaCoordinador) {
                //se va a enviar al decanato
                $oDato->setEstado(TFlujo::K_DECANATO);
                $oDato->setTexto($sTexto);
                $oDato->setRespuesta(TFlujo::K_VALIDA_OD);
            } elseif ($bEsCalidad) {
                //se va a enviar a CALIDAD
                $oDato->setEstado(TFlujo::K_CALIDAD);
                $oDato->setTexto($sTexto);
                $oDato->setRespuesta(TFlujo::K_VALIDA_OD);
            } elseif ($bEnviaRector) {
                //Rector/calidad valida la O.D, con lo que regresa a COORDINADOR por si hubiese futuros cambios
                $oDato->setEstado(TFlujo::K_COORDINA);
                $oDato->setTexto($sTexto);
                $oDato->setRespuesta(TFlujo::K_VALIDA_OD);
            } elseif ($bRechazar) {
                //Rector/calidad rechaza la O.D, con lo que regresa a COORDINADOR 
                $oDato->setEstado(TFlujo::K_COORDINA);
                $oDato->setTexto($sTexto);
                $oDato->setRespuesta(TFlujo::K_RECHAZA_OD);
            }

            //el fichero siempre son enviados por decanato
            if ($fichero) {
                //si el decano va a enviar a calidad, ya se han rellenado correctamente los campos
                if (!$bEsCalidad) {
                    //si ha validado las asignaturas y los grupos ponemos esFinal =S ->  R, para indicar que puede validar la O.D.
                    $oDato->setEstado($esFinal == 'S' ? TFlujo::K_RECTORADO :  TFlujo::K_RECTORADO . $nTipo);
                    $oDato->setTexto($sTexto);
                    $oDato->setRespuesta(TFlujo::K_VALIDA_OD);
                }
                $oDato->setArchivo($fichero);
            }

            $this->grabarObjetoBD($oDato, $sError);

            return true;
        } catch (Exception $e) {
            throw new \Exception($e);
            $sError = $e->getMessage();
            return false;
        }
    }

    /****************************************
     * FUNCIONES Y SENTENECIAS DE PG-SQL
     * 
     */

    //Cambia toda la carga docente de un PDI por otro    
    public function sustituyePDI($oldPdi, $newPdi, $nCurso, &$sError)
    {
        try {
            // En tu controlador o servicio        
            $conn = $this->entityManager->getConnection();

            $query = "UPDATE unam_ga.asignatura_pdi_t SET docente_id = $newPdi WHERE docente_id = $oldPdi and curso_id = $nCurso";
            $affectedRows = $conn->executeStatement($query);
            self::depurar($query);
            // $affectedRows contiene el número de filas afectadas por la actualización
            return $affectedRows;
        } catch (Exception $e) {
            throw new \Exception($e);
            $sError = $e->getMessage();
            return false;
        }
    }

    //listado con los docentes  
    public function obtenerTodosDocentes($nCursoAcada)
    {
        $sql = "SELECT calculado.codigo, calculado.docente, calculado.jornada, calculado.nivel, calculado.total from (
            select pat.id codigo,
            (SELECT pt.dni|| ' - ' || pt.nombre || ' ' || pt.apellido1 || ' ' || pt.apellido2  FROM unam_gp.personal_t pt WHERE pat.personal_id = pt.id ) docente,
            (SELECT jt.nombre FROM  unam_gp.jornada_t jt WHERE pat.jornada_id = jt.id) jornada,
            (SELECT nt.apodo FROM unam_gp.nivel_t nt WHERE pat.nivel_id = nt.id ) nivel,
            unam_ga.sumarcreditosdocente(pat.personal_id, $nCursoAcada  ) as total            
             from unam_gp.personal_alta_t pat  where pat.fecha_baja is null or pat.fecha_baja > CURRENT_TIMESTAMP and pat.tipo = 1 ) calculado 
            -- where calculado.jornada <> 'TP'
             group by calculado.codigo, calculado.docente, calculado.jornada, calculado.nivel, calculado.total             
            ";
        return $this->ejecutarSQLDirecto($sql, $sError);
    }


    //calcular el total de docentes asignados por niveles
    public function calcularTotalDoctores($cursoId, $tituloId, $planId, $modalidadID, $bDoctores, $sJornada, $nFacultad, &$sError, $bReal = false)
    {
        $sWhere = " where dv.cursoacademicoid = $cursoId "; // siempre se filtra por el curso
        $sWhere .= $tituloId ? " and dv.tituloid = $tituloId " : ''; //filtrado por titulac.
        $sWhere .= $planId ? " and dv.planid = $planId " : ''; // filtramos por plan
        $sWhere .= $modalidadID ? " and dv.modalidadid = $modalidadID " : ''; // filtramos por modalidad
        $sWhere .= $sJornada ? " and dv.jornada = '$sJornada' " : ''; //filtramos por jornada
        //si es un coordinador, solamente mostramos los datos de su facultad
        $sWhere .= $nFacultad ? " and dv.tituloid in ( select tt.titulo_id from unam_ga.titulacion_t tt where tt.facultad_id = $nFacultad )  " : '';
        $sWhere .= $bReal ? " and dv.docenteid not in ( select pt.id from unam_gp.personal_t pt where pt.genero ='NO' )" : '';

        $sql = "SELECT dv.nivel AS nivel,
                COUNT(distinct(dv.docenteid)) AS total,
                (COUNT(distinct(dv.docenteid)) * 100.0 / SUM(COUNT(distinct(dv.docenteid))) OVER ()) AS porcentaje 
                FROM unam_ga.detalle_v dv $sWhere GROUP BY dv.nivel order by 1 desc ";

        if ($bDoctores) {
            $sql = "SELECT salida.nivel, salida.TP, salida.TC, salida.Total, ROUND((salida.Total / SUM(salida.Total) OVER ()) * 100, 2) AS porcentaje
                    FROM (SELECT vista.nivel, SUM(vista.TP) AS TP, SUM(vista.TC) AS TC, SUM(vista.TP + vista.TC) AS Total
                    FROM (SELECT dv.nivel, 0 as TP, COUNT(DISTINCT dv.docenteid) as TC FROM unam_ga.detalle_v dv $sWhere and dv.jornada='TC' GROUP BY dv.nivel
                        UNION
                        SELECT dv.nivel, COUNT(DISTINCT dv.docenteid) as TP, 0 as TC FROM unam_ga.detalle_v dv $sWhere and dv.jornada='TP' GROUP BY dv.nivel) vista
                    GROUP BY vista.nivel ORDER BY vista.nivel desc ) salida";
        }

        return $this->ejecutarSQLDirecto($sql, $sError);
    }

    //calcular el total de cre. asignados a cada nivel de contrato
    public function calcularCreditosDoctores($cursoId, $tituloId, $planId, $modalidadID, $bDoctores, $nFacultad, &$sError, $bReal = false)
    {
        //curso acada y asignaturas que no son TFT ni PAE
        $sWhere = " where dv.cursoacademicoid = $cursoId  ";
        $sWhere .= $tituloId ? " and dv.tituloid = $tituloId " : '';
        $sWhere .= $planId ? " and dv.planid = $planId " : '';
        $sWhere .= $modalidadID ? " and dv.modalidadid = $modalidadID " : '';
        //si es un coordinador, solamente mostramos los datos de su facultad
        $sWhere .= $nFacultad ? " and dv.tituloid in ( select tt.titulo_id from unam_ga.titulacion_t tt where tt.facultad_id = $nFacultad )  " : '';
        $sWhere .= $bReal ? " and dv.docenteid not in ( select pt.id from unam_gp.personal_t pt where pt.genero ='NO' )" : '';

        $sql = "SELECT vis.nivel, sum(vis.total) as total, ROUND((SUM(vis.total) * 100.0) / SUM(SUM(vis.total)) OVER (), 2) AS porcentaje from (
            SELECT dv.nivel AS nivel, 
            sum(CASE WHEN dv.creditosasignatura = dv.creditosporalumno THEN dv.creditosdeldocente ELSE dv.creditosdeldocente * dv.creditosporalumno END) AS total      
                    FROM unam_ga.detalle_v dv $sWhere GROUP BY dv.nivel, dv.docenteid, dv.cursoacademicoid  ) vis group by vis.nivel order by 1 desc";
        //TODO -- NO SE RESTAN LOS CRED. DE LAS DUALES --- OJO 
        //calcular solamente para doctores 
        if ($bDoctores) {
            $sql = "SELECT salida.nivel, salida.TP, salida.TC, salida.Total, ROUND((salida.Total / SUM(salida.Total) OVER ()) * 100, 2) AS porcentaje
            FROM (        
                SELECT vista2.nivel, SUM(vista2.TP) AS TP, SUM(vista2.TC) AS TC, SUM(vista2.TP + vista2.TC) AS Total FROM (    	
                SELECT vista.nivel, sum(vista.TP) TP, sum(vista.TC) TC from (
                select dv.nivel, 0 as TP,  sum(CASE WHEN dv.creditosasignatura = dv.creditosporalumno THEN dv.creditosdeldocente ELSE dv.creditosdeldocente * dv.creditosporalumno END)
                 as TC from unam_ga.detalle_v dv $sWhere and  dv.jornada='TC' group by dv.nivel,dv.docenteid, dv.cursoacademicoid 
                union all
                select dv.nivel, sum(CASE WHEN dv.creditosasignatura = dv.creditosporalumno THEN dv.creditosdeldocente ELSE dv.creditosdeldocente * dv.creditosporalumno END) as TP, 0 as TC from unam_ga.detalle_v dv $sWhere and  dv.jornada='TP' group by dv.nivel,dv.docenteid, dv.cursoacademicoid 
          ) vista group by vista.nivel order by 1 desc  ) vista2 GROUP BY vista2.nivel ORDER BY vista2.nivel desc ) salida";
        }

        return $this->ejecutarSQLDirecto($sql, $sError);
    }

    //calcular el total de horas
    public function calcularHorasDoctores($cursoId, $tituloId, $planId, $modalidadID, $bDoctores, $nFacultad, &$sError, $bReal = false)
    {
        $sWhere = " where dv.cursoacademicoid = $cursoId ";
        $sWhere .= $tituloId ? " and dv.tituloid = $tituloId " : '';
        $sWhere .= $planId ? " and dv.planid = $planId " : '';
        $sWhere .= $modalidadID ? " and dv.modalidadid = $modalidadID " : '';
        //si es un coordinador, solamente mostramos los datos de su facultad
        $sWhere .= $nFacultad ? " and dv.tituloid in ( select tt.titulo_id from unam_ga.titulacion_t tt where tt.facultad_id = $nFacultad )  " : '';
        $sWhere .= $bReal ? " and dv.docenteid not in ( select pt.id from unam_gp.personal_t pt where pt.genero ='NO' )" : '';

        $horaPAE = TAsignaturaCurso::K_HORAS_ALUMNO_PAE;
        $horaTFT = TAsignaturaCurso::K_HORAS_ALUMNO_TFT;

        $sql = "SELECT Calculos.nivel,  Calculos.total, (Calculos.total * 100.0 / SUM(Calculos.total) OVER ()) AS porcentaje
        FROM (SELECT  dv.nivel AS nivel,
                SUM(CASE 
                        WHEN dv.creditosasignatura = dv.creditosporalumno THEN (dv.horasasignatura / dv.creditosasignatura) * dv.creditosdeldocente 
                        WHEN dv.creditosasignatura <> dv.creditosporalumno AND dv.tipoasignatura = 'PAE' THEN $horaPAE * dv.creditosdeldocente
                        WHEN dv.creditosasignatura <> dv.creditosporalumno AND dv.tipoasignatura <> 'PAE' THEN $horaTFT * dv.creditosdeldocente
                        ELSE 0 
                    END) AS total
            FROM unam_ga.detalle_v dv  $sWhere GROUP BY dv.nivel ) AS Calculos order by 1 desc";


        if ($bDoctores) {
            //calcular solamente para doctores -- 
            $sql = "SELECT salida.nivel, salida.TP, salida.TC, salida.Total, ROUND((salida.Total / SUM(salida.Total) OVER ()) * 100, 2) AS porcentaje
                    FROM ( SELECT vista1.nivel, SUM(vista1.TP) AS TP, SUM(vista1.TC) AS TC, SUM(vista1.TP + vista1.TC) AS Total
                    FROM (SELECT vista.nivel, sum(vista.TP) TP, sum(vista.TC) TC from (
                                    select dv.nivel, 0 as TP,   SUM(CASE 
                                                            WHEN dv.creditosasignatura = dv.creditosporalumno THEN (dv.horasasignatura / dv.creditosasignatura) * dv.creditosdeldocente 
                                                            WHEN dv.creditosasignatura <> dv.creditosporalumno AND dv.tipoasignatura = 'PAE' THEN $horaPAE * dv.creditosdeldocente
                                                            WHEN dv.creditosasignatura <> dv.creditosporalumno AND dv.tipoasignatura <> 'PAE' THEN $horaTFT * dv.creditosdeldocente
                                                            ELSE 0 
                                                        END) as TC from unam_ga.detalle_v dv $sWhere and  dv.jornada='TC' group by dv.nivel union
                                    select dv.nivel,  SUM(CASE 
                                                            WHEN dv.creditosasignatura = dv.creditosporalumno THEN (dv.horasasignatura / dv.creditosasignatura) * dv.creditosdeldocente 
                                                            WHEN dv.creditosasignatura <> dv.creditosporalumno AND dv.tipoasignatura = 'PAE' THEN $horaPAE * dv.creditosdeldocente
                                                            WHEN dv.creditosasignatura <> dv.creditosporalumno AND dv.tipoasignatura <> 'PAE' THEN $horaTFT * dv.creditosdeldocente
                                                            ELSE 0 
                                                        END) as TP, 0 as TC from unam_ga.detalle_v dv $sWhere and  dv.jornada='TP' group by dv.nivel 
                                    ) vista group by vista.nivel order by 1 desc ) vista1
                    GROUP BY vista1.nivel
                    ORDER BY vista1.nivel desc  ) salida";
        }

        return $this->ejecutarSQLDirecto($sql, $sError);
    }

    //el total de docentes asignados por tipo de jornada
    public function calcularJornada($cursoId, $tituloId, $planId, $modalidadID, $nFacultad, $sCreditos, &$sError, $bReal = false)
    {

        $sWhere = " where dv.cursoacademicoid = $cursoId ";
        $sWhere .= $tituloId ? " and dv.tituloid = $tituloId " : '';
        $sWhere .= $planId ? " and dv.planid = $planId " : '';
        $sWhere .= $modalidadID ? " and dv.modalidadid = $modalidadID " : '';
        $sWhere .= $bReal ? " and dv.docenteid not in ( select pt.id from unam_gp.personal_t pt where pt.genero ='NO' )" : '';
        //si es un coordinador, solamente mostramos los datos de su facultad
        $sWhere .= $nFacultad ? " and dv.tituloid in ( select tt.titulo_id from unam_ga.titulacion_t tt where tt.facultad_id = $nFacultad )  " : '';
        $jornadaCompleta = TJornada::K_JORNADA_COMPLETA;
        $horaPAE = TAsignaturaCurso::K_HORAS_ALUMNO_PAE;
        $horaTFT = TAsignaturaCurso::K_HORAS_ALUMNO_TFT;
        //obtenemos los docentes equivalentes totales
        $sql = "SELECT salida.nivel, salida.equivalente, ROUND((salida.equivalente / SUM(salida.equivalente) OVER ()) * 100, 2) AS porcentaje
                FROM (
                    SELECT eq.nivel, SUM(eq.completo) AS equivalente
                    FROM (
                        SELECT DISTINCT dv.docenteid, dv.nivel, coalesce(dv.horasdocente::numeric, $jornadaCompleta ) / $jornadaCompleta AS completo
                        FROM unam_ga.detalle_v dv $sWhere
                    ) eq
                    GROUP BY eq.nivel
                    ORDER BY eq.nivel DESC) salida";


        if ($sCreditos == 'C') {
            //obtenemos los cre. equivalentes totales
            $sql = "SELECT salida.nivel, salida.equivalente, 
            ROUND((salida.equivalente / SUM(salida.equivalente) OVER ()) * 100, 2) AS porcentaje
                from (
            SELECT dv.nivel AS nivel, 
            SUM(CASE WHEN dv.creditosasignatura = dv.creditosporalumno THEN coalesce(dv.horasdocente::numeric, $jornadaCompleta )/$jornadaCompleta  * dv.creditosdeldocente ELSE coalesce(dv.horasdocente::numeric, $jornadaCompleta )/$jornadaCompleta * (dv.creditosdeldocente * dv.creditosporalumno) END) AS equivalente        
            FROM unam_ga.detalle_v dv $sWhere GROUP BY dv.nivel order by 1 desc ) salida ";
        } elseif ($sCreditos == 'H') {
            //obtenemos las horas equivalentes totales
            $sql = "SELECT salida.nivel, salida.equivalente, 
            ROUND((salida.equivalente / SUM(salida.equivalente) OVER ()) * 100, 2) AS porcentaje
     from (SELECT Calculos.nivel,  Calculos.total equivalente
            FROM (SELECT  dv.nivel AS nivel,
                    SUM(CASE 
                            WHEN dv.creditosasignatura = dv.creditosporalumno THEN (dv.horasasignatura / dv.creditosasignatura) * dv.creditosdeldocente * coalesce(dv.horasdocente::numeric, $jornadaCompleta )/$jornadaCompleta 
                            WHEN dv.creditosasignatura <> dv.creditosporalumno AND dv.tipoasignatura = 'PAE' THEN $horaPAE * dv.creditosdeldocente * coalesce(dv.horasdocente::numeric, $jornadaCompleta )/$jornadaCompleta
                            WHEN dv.creditosasignatura <> dv.creditosporalumno AND dv.tipoasignatura <> 'PAE' THEN $horaTFT * dv.creditosdeldocente * coalesce(dv.horasdocente::numeric, $jornadaCompleta )/$jornadaCompleta
                            ELSE 0 
                        END) AS total
                FROM unam_ga.detalle_v dv $sWhere GROUP BY dv.nivel ) AS Calculos order by 1 desc ) salida ";
        }

        return $this->ejecutarSQLDirecto($sql, $sError);
    }

    // Obtener los datos de los docentes por titulac.
    public function calcularDatosDocentes($cursoId, $tituloId, $planId, $modalidadID, $nFacultad, &$sError)
    {
        $sWhere = ''; // " where dv.cursoacademicoid = $cursoId ";
        $sWhere .= $tituloId ? " and dv.tituloid = $tituloId " : '';
        $sWhere .= $planId ? " and dv.planid = $planId " : '';
        $sWhere .= $modalidadID ? " and dv.modalidadid = $modalidadID " : '';
        //si es un coordinador, solamente mostramos los datos de su facultad
        $sWhere .= $nFacultad ? " and dv.tituloid in ( select tt.titulo_id from unam_ga.titulacion_t tt where tt.facultad_id = $nFacultad )  " : '';

        //OJO -- TODO -- SI TIENE + de un contrato hay q localizar el correcto
       /* $sql = "SELECT (select pat.id from unam_gp.personal_alta_t pat where pat.personal_id =dv.docenteid) docenteid ,dv.docente ,dv.titulo, 
        SUM( CASE  WHEN dv.creditosasignatura = dv.creditosporalumno THEN dv.creditosdeldocente ELSE dv.creditosdeldocente * dv.creditosporalumno END) AS total_creditos,
        ROUND((SUM(CASE WHEN dv.creditosasignatura = dv.creditosporalumno THEN dv.creditosdeldocente ELSE dv.creditosdeldocente * dv.creditosporalumno END
            ) / SUM(SUM(CASE WHEN dv.creditosasignatura = dv.creditosporalumno THEN dv.creditosdeldocente ELSE dv.creditosdeldocente * dv.creditosporalumno END
            )) OVER (PARTITION BY dv.docente)) * 100, 2
        ) AS porcentaje FROM unam_ga.detalle_v dv $sWhere GROUP BY dv.docenteid ,dv.docente, dv.titulo ORDER BY dv.docente";*/

        $sql = "SELECT dv.contratoActual docenteid ,dv.docente ,dv.titulo ,
        SUM( CASE  WHEN dv.creditosasignatura = dv.creditosporalumno THEN dv.creditosdeldocente ELSE dv.creditosdeldocente * dv.creditosporalumno END) AS total_creditos,
        ROUND((SUM(CASE WHEN dv.creditosasignatura = dv.creditosporalumno THEN dv.creditosdeldocente ELSE dv.creditosdeldocente * dv.creditosporalumno END
            ) / SUM(SUM(CASE WHEN dv.creditosasignatura = dv.creditosporalumno THEN dv.creditosdeldocente ELSE dv.creditosdeldocente * dv.creditosporalumno END
            )) OVER (PARTITION BY dv.docente)) * 100, 2
        ) AS porcentaje  
        FROM unam_ga.detalle_v dv where dv.fechaBaja is null or dv.fechaBaja > CURRENT_TIMESTAMP $sWhere
        GROUP BY dv.docenteid ,dv.docente, dv.titulo,  dv.contratoActual ORDER BY dv.docente;";
        //self::depurar($sql);
        $aDatos['tabla'] = $this->ejecutarSQLDirecto($sql, $sError);

        // Obtener todas las titulaciones únicas
        $titulaciones = [];
        foreach ($aDatos['tabla'] as $dato) {
            $titulaciones[$dato['titulo']] = $dato['titulo'];
        }

        $aDatos['titulos'] = $titulaciones;
        $pivotado = [];
        foreach ($aDatos['tabla'] as $dato) {

            $docente = $dato['docente'];
            $titulo = $dato['titulo'];
            $porcentaje = $dato['porcentaje'];

            // Verificar si el docente ya existe en el array pivotado
            if (!isset($pivotado[$docente])) {
                $pivotado[$docente] = ['docente' => explode('-', $docente)[1]];
            }
            //$pivotado[$docente][$titulo] = $porcentaje;

            foreach ($titulaciones as $tituloFor) {
                $valor = $porcentaje;
                if ($titulo == $tituloFor) {
                    $pivotado[$docente][$tituloFor] = $valor;
                } else {
                    $pivotado[$docente][$tituloFor] = isset($pivotado[$docente][$tituloFor]) && $pivotado[$docente][$tituloFor] > 0 ? $pivotado[$docente][$tituloFor] : 0;
                }
            }
        }

        $aDatos['filas'] = $pivotado;
        return $aDatos;
    }

    //obtenemos el listado de docencia por departamento de asignaturas asignadas al PDI
    public function obtenerPdiDpto($cursoId)
    {
        $nSinDepartamento = 5; //valor en la BD del dpto "sin dpto" - OJO 
       /* $sql = "SELECT datos.creditos, 
            (select dt.nombre from unam_ga.departamento_t dt where dt.id = datos.dpto) dpto,
            (select pt.dni ||' - '|| pt.nombre || ' '||pt.apellido1 ||' '||pt.apellido2  from unam_gp.personal_alta_t pat , unam_gp.personal_t pt where  pat.personal_id = pt.id and   pat.id= datos.docente_id) docente
        from (
            select  coalesce (at2.departamento_id, $nSinDepartamento) dpto , apt.docente_id , sum(apt.creditos)  creditos
            from unam_ga.asignatura_pdi_t apt , unam_ga.asignatura_curso_t act , unam_ga.asignatura_t at2 
            where apt.asignatura_id  = act.id  and at2.id = act.asignatura_id 
            and apt.curso_id = $cursoId group by  at2.departamento_id , apt.docente_id) datos";*/

        $sql = "SELECT sum(datos.creditos) creditos, 
            (select dt.nombre from unam_ga.departamento_t dt where dt.id = datos.dpto) dpto,
            (select pt.dni ||' - '|| pt.nombre || ' '||pt.apellido1 ||' '||pt.apellido2  
             from unam_gp.personal_alta_t pat , unam_gp.personal_t pt where  pat.personal_id = pt.id and   pat.id= datos.docente_id) docente
        from (            
        	select  dpto , docente_id , 
case when conDual.dual = 'S' then sum(creditos)/2 else sum(creditos) end creditos from (
select coalesce (at2.departamento_id, $nSinDepartamento) dpto , apt.docente_id , apt.creditos,
case when act.id  in (
select adt.asignatura_id asignatura from unam_ga.asignatura_dual_t adt union
select adt.asignatura_dual asignatura from unam_ga.asignatura_dual_t adt 
) and apt.grupo > act.grupos then 'S' else 'N' end dual
            from unam_ga.asignatura_pdi_t apt , unam_ga.asignatura_curso_t act , unam_ga.asignatura_t at2 
            where apt.asignatura_id  = act.id  and at2.id = act.asignatura_id 
            and apt.curso_id = $cursoId 
            ) conDual group by conDual.dpto, conDual.docente_id, conDual.dual           
            ) datos group by datos.dpto, datos.docente_id;";

        return $this->ejecutarSQLDirecto($sql, $sError);
    }

    //comprueba si la O.D de la facultad ya ha sido validada
    public function validadoODFacultad($cursoId, $nFacultad)
    {
        $esCalidad = Tusuarios::K_TIPO_CALIDAD;
        $esRector = Tusuarios::K_TIPO_RECTOR;

        $sql = "SELECT count(*) total from unam_ga.flujo_t ft where ft.curso_id= $cursoId and ft.facultad_id = $nFacultad and ft.estado = 'C' and ft.respuesta = 1 and ft.personal_id in (
            select ut.id from unam_ga.usuarios_t ut where ut.tipo = $esRector or ut.tipo = $esCalidad)";

        return $this->ejecutarSQLDirecto($sql, $sError);
    }

    //obtiene da la BD los datos de la SQL pasada    
    private function ejecutarSQLDirecto($sql, &$sError)
    {
        try {
            $conn = $this->entityManager->getConnection();
            //ORM::depurar($sql);
            $stmt = $conn->prepare($sql);
            $resultado = $stmt->executeQuery()->fetchAllAssociative();
        } catch (\Exception $e) {
            // Capturar y manejar la excepción
            $sError = "Error al ejecutar la función: $sql " . $e->getMessage();
            $resultado = 0;
        }
        return $resultado;
    }

    // ***************************************//

    public function obtenerCreditosDualesDocente($docente, $curso, &$sError)
    {
        try {
            $conn = $this->entityManager->getConnection();
            //usamos function de PGSQL 
            $sql = 'SELECT unam_ga.calcularCreditosDuales(:docente, :curso) AS resultado';
            $stmt = $conn->prepare($sql);
            $stmt->bindValue('docente', $docente);
            $stmt->bindValue('curso', $curso);
            $resultado = $stmt->executeQuery()->fetchOne();
        } catch (\Exception $e) {
            // Capturar y manejar la excepción
            $sError = "Error al ejecutar la función obtenerCreditosDualesDocente: " . $e->getMessage();
            $resultado = 0;
        }
        return $resultado;
    }

    //comprueba si una asignatura existe en la tabla de relaciones duales, en caso afirmativo, retorna el codAsignatura de su dual
    public function obtenerSiEsAsignaturaDual($idAsignatura, &$sError)
    {
        try {
            $conn = $this->entityManager->getConnection();
            //usamos function de PGSQL 
            $sql = 'SELECT unam_ga.obtenerDual(:asignatura) AS resultado';
            $stmt = $conn->prepare($sql);
            $stmt->bindValue('asignatura', $idAsignatura);
            $resultado = $stmt->executeQuery()->fetchOne();
        } catch (\Exception $e) {
            // Capturar y manejar la excepción
            $sError = "Error al ejecutar la función obtenerDual: " . $e->getMessage();
            $resultado = 0;
        }
        return $resultado;
    }

    //obtiene la suma de los cre. asignados en el curso acada. pasado por param.
    public function sumarCreditosDocente($docente, $ncurso)
    {
        $conn = $this->entityManager->getConnection();
        $sql = "SELECT unam_ga.sumarCreditosDocente($docente, $ncurso) AS resultado";
        $stmt = $conn->prepare($sql);
        $resultadoTotal = $stmt->executeQuery()->fetchOne();
       // self::depurar("$docente, $ncurso, $resultadoTotal");
        return $resultadoTotal > 0 ? number_format($resultadoTotal, 2) : 0;
    }

    //obtiene la suma de los cre. asignados en el curso acada. pasado por param.
    public function sumarCreditosAsignatura($asignatura, $grupo, $ncurso)
    {
        $conn = $this->entityManager->getConnection();
        //usamos function de PGSQL 
        $sql = 'SELECT unam_ga.sumarCreditosAsignatura(:asignatura, :grupo, :curso) AS resultado';
        $stmt = $conn->prepare($sql);
        $stmt->bindValue('asignatura', $asignatura);
        $stmt->bindValue('curso', $ncurso);
        $stmt->bindValue('grupo', $grupo);
        $resultado = $stmt->executeQuery()->fetchOne();        
        return $resultado;
    }


    //Dada una asignatura, calcula su dual para guardarla en la tabla
    public function obtenerasignaturadual($asignatura, $titulo, $ncurso, &$sError)
    {
        try {
            $conn = $this->entityManager->getConnection();
            //usamos function de PGSQL 
            $sql = 'SELECT unam_ga.obtenerasignaturadual(:asignatura, :titulo, :curso) AS resultado';
            $stmt = $conn->prepare($sql);
            $stmt->bindValue('asignatura', $asignatura);
            $stmt->bindValue('curso', $ncurso);
            $stmt->bindValue('titulo', $titulo);
            $resultado = $stmt->executeQuery()->fetchOne();
        } catch (\Exception $e) {
            // Capturar y manejar la excepción
            $sError = "Error al ejecutar la función obtenerasignaturadual: " . $e->getMessage();
            $resultado = 0;
        }

        return $resultado;
    }

    //comprueba si es una asignatura DUAL
    //OJO al migrar los datos el PLAN ESPECIAL es el que indica que asignaturas pueden ser DUAL
    //si cambia ese PLAN no funciona
    public function comprobarasignaturadual($asignatura, &$sError)
    {
        try {
            $conn = $this->entityManager->getConnection();
            //usamos function de PGSQL              
            $sql = 'SELECT unam_ga.comprobarDual(:asignatura) AS resultado';
            $stmt = $conn->prepare($sql);
            $stmt->bindValue('asignatura', $asignatura);
            $resultado = $stmt->executeQuery()->fetchOne();
        } catch (\Exception $e) {
            // Capturar y manejar la excepción
            $sError = "Error al ejecutar la función comprobarasignaturadual: " . $e->getMessage();
            $resultado = 0;
        }

        return $resultado;
    }



    /*********************** */
    //localizar si una asignatura es dual
    public function buscarAsignaturaDual($valorBuscado)
    {
        $query = $this->entityManager->createQuery(
            'SELECT e
            FROM App\Entity\TAsignaturaDual e
            WHERE e.asignatura = :valorBuscado OR e.asignaturaDual = :valorBuscado'
        )->setParameter('valorBuscado', $valorBuscado);

        $resultados = $query->getResult();

        return $resultados;
    }

    public function obtenerAsignaturasTitulacionCurso($titulo, $nModalidad, $nPlan, $ncurso, $ntipo)
    {
        //ORM::depurar(func_get_args());
        $dql = "SELECT act, at2
        FROM App\Entity\TAsignaturaCurso act
        JOIN act.asignatura at2
        WHERE at2.modalidad = :modalidadId
        AND at2.plan = :planId
        AND at2.tipoAsignatura = :tipoId
        AND at2.titulacion = :titulacionId
        AND act.cursoAcada = :cursoId";

        $query = $this->entityManager->createQuery($dql)
            ->setParameter('modalidadId', $nModalidad)
            ->setParameter('planId', $nPlan)
            ->setParameter('titulacionId', $titulo)
            ->setParameter('cursoId', $ncurso)
            ->setParameter('tipoId', $ntipo);

        return $query->getResult();
    }

    /************************************************************
     * AUXILIARES
     */


    /**
     * @param $object
     * @param $sError
     * @return bool
     */
    public function grabarObjetoBD(&$object, &$sError)
    {

        try {

            if (!is_object($object) or ($object == NULL)) {
                throw new \Exception('Imposible grabar, datos vacíos');
            }

            $this->entityManager->persist($object);
            $this->entityManager->flush();

            return TRUE;
        } catch (\Exception $e) {
            throw new \Exception($e);
            //\Doctrine\Common\Util\Debug::dump ( $object );              
            return FALSE;
        }
    }

    /************************************************************/

    /**
     * @param $object
     * @param $sError
     * @return bool
     */
    private function borrarObjetoBD(&$object, &$sError)
    {

        try {

            if (!is_object($object) or ($object == NULL)) {
                throw new \Exception('Imposible eliminar, datos vacíos');
            }

            $this->entityManager->remove($object);
            $this->entityManager->flush();

            return TRUE;
        } catch (\Exception $e) {
            throw new \Exception($e);
            return FALSE;
        }
    }

    /************************************************************/

    /**
     * @param $em
     * @param $sql
     * @return mixed
     */
    private function obtenerSQL($em, $sql)
    {
        $connection = $em->getConnection();
        $results = $connection->executeQuery($sql)->fetchAll();

        return $results;
    }

    /************************************************************/
    /**
     * Muestra por pantalla el valor de la variable
     */
    static function depurar($valor, $doctrine = false, $die = false)
    {
        if ($doctrine) {
            echo "<pre>";
            \Doctrine\Common\Util\Debug::dump($valor);
            echo "</pre>";
        } else {
            echo "<pre>";
            print_r($valor);
            echo "</pre>";
        }

        if ($die) {
            exit();
            die();
        }
    }
} // de la clase