<?php

/**
 * Clase/servicio que implementa metodos para que las diferentes plantillas las puedan invocar y así refactorizar, desacoplar y quitarle
 * lógica a éstas
 */

namespace App\Service;

use App\Entity\TAsignaturaCurso;
use App\Entity\TAsignaturaPdi;
use Twig\TwigFunction;

use Twig\Extension\AbstractExtension;
use App\ORM\ORM;
use Twig\TwigFilter;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;
use Symfony\Component\DependencyInjection\ParameterBag\ParameterBagInterface;
use Doctrine\ORM\EntityManagerInterface;
use Endroid\QrCode\Color\Color;
use Endroid\QrCode\Encoding\Encoding;
use Endroid\QrCode\ErrorCorrectionLevel\ErrorCorrectionLevelLow;
use Endroid\QrCode\QrCode;
use Endroid\QrCode\Label\Label;
use Endroid\QrCode\Logo\Logo;
use Endroid\QrCode\Writer\PngWriter;
use Endroid\QrCode\Label\Font\NotoSans;

class extraTwigExtension  extends AbstractExtension
{
    private $rutaFichero;
    private $orm;

    public function __construct(ParameterBagInterface $params, EntityManagerInterface $entityManager)
    {
        $this->orm = new ORM($entityManager);
        $this->rutaFichero = $params->get('rutaDirectorio');
    }

    /**Hay que registrar los metodos que se vayan a invocar desde la plantillas. El primer parametro es el nombre del metodo para la
     * plantilla, y el segundo es el nombre del metodo en la clase (pueden ser iguales)
     */
    public function getFunctions()
    {
        return [
            new TwigFunction('area', [$this, 'calculateArea']),
            new TwigFunction('fileExist', [$this, 'fileExist']),
            new TwigFunction('obtenerQR', [$this, 'obtenerQR']),
            new TwigFunction('creditosAsignados', [$this, 'creditosAsignados']),
            new TwigFunction('creditosAsignadosActualmente', [$this, 'creditosAsignadosActualmente']),
            new TwigFunction('creditosAsignadosAsignatura', [$this, 'creditosAsignadosAsignatura']),
            new TwigFunction('creditosDualesDocente', [$this, 'creditosDualesDocente']),
        ];
    }

    /**
     * Para cada asignatura, obtiene los cre. asignados
     */
    public function creditosDualesDocente($nIdDocente, $nCurso)
    {
        $nTotalAsignado =  $this->orm->obtenerCreditosDualesDocente($nIdDocente,  $nCurso, $sError);
        return $nTotalAsignado;
    }

    /**
     * Para cada asignatura, obtiene los cre. asignados
     */
    public function creditosAsignadosAsignatura($nIdAsignaturaCurso, $ngrupo, $nCurso)
    {
        $nTotalAsignado =  $this->orm->sumarCreditosAsignatura($nIdAsignaturaCurso, $ngrupo, $nCurso);
        return $nTotalAsignado;
    }

    /**
     * 
     */
    public function creditosAsignadosActualmente($nIdAsignaturaCurso)
    {
        $aAsignaturaCurso =  $this->orm->obtenerDocentesAsignatura($nIdAsignaturaCurso);
        $nTotalAsignado = 0;

        foreach ($aAsignaturaCurso as $docentes) {
            $creditosDocente = is_numeric( $docentes->getCreditos() ) ? $docentes->getCreditos() : 0;
            $nTotalAsignado += $creditosDocente ;
        }

        return $nTotalAsignado;
    }

    /**
     * Para una asignatura se comprueba si ya tiene toda la docencia asignada
     */
    public function creditosAsignados($nIdAsignaturaCurso)
    {

        $oAsignaturaCurso =  $this->orm->obtenerEntidad($nIdAsignaturaCurso, TAsignaturaCurso::K_ENTIDAD);
        $nGrupos = $oAsignaturaCurso->getGrupos();
        $nCreditos = $oAsignaturaCurso->getAsignatura()->getCreditos();

        $nTotalAsignado = $this->creditosAsignadosActualmente($nIdAsignaturaCurso);
        $nTotal = $nGrupos * $nCreditos;

        //+ docencia que cre.
        if ($nTotalAsignado > $nTotal) {
            return -1;
        }
        //sin docencia asignada
        elseif ($nTotalAsignado == 0) {
            return 1;
            //falta docencia por asignar  
        } elseif ($nTotalAsignado < $nTotal) {
            return 2;
        } else {
            //docencia completa
            return 3;
        }
    }

    public function calculateArea(int $width, int $length)
    {
        return $width * $length;
    }

    public function fileExist($ruta)
    {
        return file_exists($this->rutaFichero . $ruta);
    }

    public function obtenerQR($id, $ruta)
    {
        $url = $ruta . $id; // Reemplaza esto con tu URL

        $writer = new PngWriter();
        $label = Label::create('')->setFont(new NotoSans(8));
        $qrCode = QrCode::create($url)
            ->setEncoding(new Encoding('UTF-8'))
            ->setErrorCorrectionLevel(new ErrorCorrectionLevelLow())
            ->setSize(60)
            ->setMargin(0)
            ->setForegroundColor(new Color(0, 0, 0))
            ->setBackgroundColor(new Color(255, 255, 255));

        $QR = $writer->write($qrCode, null, $label->setText('UNAM'))->getDataUri();

        return $QR;
    }
}
